#!/usr/bin/env python

import json
import sys
import subprocess

def runCommand(cmd):
    proc = subprocess.Popen(cmd,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE,
                            shell=True,
                            universal_newlines=True)
    std_out, std_err = proc.communicate()
    return proc.returncode, std_out, std_err

def executeCommands(commands):
    for cmd in commands:
        print("Execute: %s" % cmd)
        ret_code, out, err = runCommand(cmd)
        print("stdout: %s" % out)
        print("stderr: %s" % err)
        if ret_code != 0 or err != "" or err.strip() != "":
            print("Command failed: %s, out = %s, err = %s" % (cmd, out, err))
            sys.exit(1)

def main(args):
    gems_file=args[0]
    profiles_str=args[1]
    profiles=profiles_str.split(",")
    gems_content=None
    commands=list()
    with open(gems_file, 'r') as gf:
        gems_content=json.load(gf)
    if gems_content:
        gems=gems_content["gems"]
        for obj in gems:
            commands.append("gem fetch %s --version %s" % (obj["name"], obj["version"]))
        plugin_gems=gems_content["plugin_gems"]
        for plugin_obj in plugin_gems:
            if "profile" in plugin_obj and not plugin_obj["profile"] in profiles:
                continue
            cmd="gem fetch %s --version %s" % (plugin_obj["name"], plugin_obj["version"])
            if "repo" in plugin_obj:
                cmd+=" -s %s" % plugin_obj["repo"]
            commands.append(cmd)
    executeCommands(commands)

if __name__ == "__main__":
    main(sys.argv[1:])