# CDP Telemetry CLI

[![Build Status](https://build.service-delivery.cloudera.com/job/cdp-telemetry-cli-build/badge/icon)](https://build.service-delivery.cloudera.com/job/cdp-telemetry-cli-build)

CLI tool for telemetry related operations on CDP clusters. (FreeIPA/DL/DH)

## Components

- [CDP doctor](docs/CDP_DOCTOR.md)
- [CDP nodestatus-monitor](docs/CDP_NODESTATUS_MONITOR.md)
- [Filecollector](docs/FILECOLLECTOR.md)
- [Storage Client](docs/STORAGE.md)
- [Databus Client](docs/DATABUS.md)
- metrics collector: agent for collecting metrics from CM or FreeIPA (deprecated - as prometheus/request-signer is used for that, but some of it's code can be re-used later for checks that should communicate with CM - as it contains CM python client)

## Development

### Requirements

- Python 3.5+
- pip3
- docker

### Usage

Run and package application locally:

```bash
# setup virtual env
python3 -m venv env1
source env1/bin/activate
# create package - generate api clients
make package
# run cli application
python3 cdp_telemetry/cli.py
```

Build RPM with docker (fpm):

```bash
make linux-rpm
```

Testing RPM based docker image:

```bash
docker run --rm cloudera/cdp-telemetry --help
```

Re-generate DBus client (if required):

```bash
make generate-api-clients
```

Re-generate protobuf models (for nodestatus monitor)

```bash
make generate-proto-models
```

### Notes

- CDP telemetry is built by [pyinstaller](https://pyinstaller.org/en/stable/) in order to create a binary instead of relying on python versions
- `filecollector` which is part of cdp-telemetry is also a standalone binary (written in `Go`), and it is available next to `cdp-telemetry` binary

## Publishing releases

1. Create a new tag (can be done on master if the code changes already pushed to remote):

```bash
make tag
```

2. Rebase cdp-telmetry-release branch on master and push that to remote (so the tag will be available on the branch)

```bash
git checkout cdp-telemetry-release
git rebase origin/master
git push origin cdp-telemetry-release
```

2. Last step will trigger [this](https://build.service-delivery.cloudera.com/job/cdp-telemetry-cli-release/) jenkins job.

4. Increase value in VERSION file on `master` branch

Notes: rpm will be uploading to:s

Snapshot: https://cloudera-service-delivery-cache.s3.amazonaws.com/telemetry/cdp-telemetry/cdp_telemetry-0.1.0-SNAPSHOT.x86_64.rpm

Release: https://cloudera-service-delivery-cache.s3.amazonaws.com/telemetry/cdp-telemetry/cdp_telemetry-{release_version}.x86_64.rpm

## Test Snapshot Build

In order to test any master build (before release or for validation etc.) on VMs, you can simply do the following to update RPM to the latest master one:

```bash
yum remove -y cdp-telemetry
rpm -i https://cloudera-service-delivery-cache.s3.amazonaws.com/telemetry/cdp-telemetry/cdp_telemetry-0.1.0-SNAPSHOT.x86_64.rpm
```

## License

Apache 2.0
