import yaml
import os
import logging
from jinja2 import Environment, Template
from utils import telemetry_logger

logger=logging.getLogger(__name__)

report_template="""
<!doctype html>
<html lang="en">
  <head>
    <!-- Required meta tags -->
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">

    <!-- Bootstrap CSS -->
    <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/bootstrap@4.5.3/dist/css/bootstrap.min.css" integrity="sha384-TX8t27EcRE3e/ihU7zmQxVncDAy5uIKz4rEkgIXeMed4M0jlfIDPvg6uqKI2xXr2" crossorigin="anonymous">

    <title>{{ templateVars["title"] }}</title>
  </head>
  <body>
    <h1>{{ templateVars["title"] }}</h1>
    <!-- Optional JavaScript; choose one of the two! -->
    <!-- Option 1: jQuery and Bootstrap Bundle (includes Popper) -->
    <script src="https://code.jquery.com/jquery-3.5.1.slim.min.js" integrity="sha384-DfXdz2htPH0lsSSs5nCTpuj/zy4C+OGpamoFVy38MVBnE+IbbVYUew+OrCXaRkfj" crossorigin="anonymous"></script>
    <script src="https://cdn.jsdelivr.net/npm/bootstrap@4.5.3/dist/js/bootstrap.bundle.min.js" integrity="sha384-ho+j7jyWK8fNQe+A12Hb8AhRq26LrZ/JpcUGGOn+Y7RsweNrtN/tE3MoK7ZeZDyx" crossorigin="anonymous"></script>
    <script src="https://cdn.jsdelivr.net/gh/google/code-prettify@master/loader/run_prettify.js"></script>
    <!-- Option 2: jQuery, Popper.js, and Bootstrap JS
    <script src="https://code.jquery.com/jquery-3.5.1.slim.min.js" integrity="sha384-DfXdz2htPH0lsSSs5nCTpuj/zy4C+OGpamoFVy38MVBnE+IbbVYUew+OrCXaRkfj" crossorigin="anonymous"></script>
    <script src="https://cdn.jsdelivr.net/npm/popper.js@1.16.1/dist/umd/popper.min.js" integrity="sha384-9/reFTGAW83EW2RDu2S0VKaIzap3H66lZH81PoYlFhbGU+6BZp6G7niu735Sk7lN" crossorigin="anonymous"></script>
    <script src="https://cdn.jsdelivr.net/npm/bootstrap@4.5.3/dist/js/bootstrap.min.js" integrity="sha384-w1Q4orYjBQndcko6MimVbzY0tgp4pWB4lZ7lr30WKz0vr/aWKhXdBNmNb5D92v7s" crossorigin="anonymous"></script>
    -->
    {% for commandOutput in templateVars["commandOutputs"] %}
    <p>Command:</p>
    <pre class="prettyprint lang-bsh">{{ commandOutput["command"] }}</pre>
    <p>Output:</p>
    <pre class="prettyprint lang-bsh">
{{ commandOutput["output"] }}
    </pre>{% endfor %}
  </body>
</html>
"""

def command_report(config_file):
    with open(config_file) as file:
        config = yaml.load(file, yaml.SafeLoader)
        commandOutputs=[]
        if "commands" in config:
            for commandObject in config["commands"]:
                logger.debug("Running command: %s > %s" % (commandObject["command"], commandObject["output"]))
                os.system("%s > %s" % (commandObject["command"], commandObject["output"]))
                if "report" not in commandObject or commandObject["report"]:
                    outputStr=telemetry_logger.remove_colors(open(commandObject["output"], 'r').read())
                    commandOutput={"command": commandObject["command"], "output": outputStr}
                    commandOutputs.append(commandOutput)
        if "report" in config:
            if "location" in config["report"] and "title" in config["report"] and commandOutputs:
                templateVars={}
                templateVars["title"]=config["report"]["title"]
                templateVars["commandOutputs"]=commandOutputs
                __render_template(config["report"]["location"], templateVars)

def __render_template(filename, templateVars):
    rendered_str=Environment().from_string(report_template).render(templateVars=templateVars)
    with open(filename, "w") as rendered_file:
        rendered_file.write(rendered_str)