#!/usr/bin/env python

import os
import logging
from utils import telemetry_logger, response

# todo: refactor this to use python
inline_script = """
#!/bin/env sh

set -e
FQDN=$(hostname -f)
echo "FQDN: $FQDN"
if [[ $OSTYPE != 'darwin'* ]]; then
  IPADDR=$(hostname -i)
  REVERSE_IP=$(hostname -i | awk -F. '{print $4"."$3"." $2"."$1}')
  NSLOOKUP_IP=$(nslookup $(hostname -i) | grep "name =" | awk '{ gsub(/name = /, " " ); print $2; }' | sed 's/\.$//')
  echo "IP: $IPADDR"
  echo "REVERSE_IP: $REVERSE_IP"
  echo "NSLOOKUP_IP: $NSLOOKUP_IP"
fi
if [[ $FQDN == ipaserver* ]]; then
  IPA_TYPE="FREEIPA"
else
  IPA_TYPE="CM_AGENT"
fi
echo "IPA_TYPE: $IPA_TYPE"

if [[ ! -d /opt/salt ]]; then
  exit 1
fi

function cleanup() {
  kdestroy
}

trap cleanup EXIT

if [[ $FQDN == ipaserver* ]]; then
  /opt/salt_*/bin/salt-call pillar.get freeipa --out=json --out-file=/tmp/ipa-doctor-pillars.json 2>/dev/null
  salt_result=$?
else
  /opt/salt_*/bin/salt-call pillar.get sssd-ipa --out=json --out-file=/tmp/ipa-doctor-pillars.json 2>/dev/null
  salt_result=$?
fi
if [[ "$salt_result" != "0" ]];then
  rm -rf /tmp/ipa-doctor-pillars.json
  exit 1
fi

if [[ $FQDN == ipaserver* ]]; then
  PW=$(cat /tmp/ipa-doctor-pillars.json | jq .local.password | tr -d '"')
  PRINCIPAL=$(cat /tmp/ipa-doctor-pillars.json| jq .local.admin_user | tr -d '"')
else
  PW=$(cat /tmp/ipa-doctor-pillars.json | jq .local.password | tr -d '"')
  PRINCIPAL=$(cat /tmp/ipa-doctor-pillars.json | jq .local.principal | tr -d '"')
fi
rm -rf /tmp/ipa-doctor-pillars.json
echo "$PW" | kinit $PRINCIPAL

ZONES=""
for zone in $(ipa dnszone-find --raw | grep "idnsname:.*\.in-addr\.arpa\." | cut -d':' -f2 | awk '{ print length, $0 }' | sort -n -r | awk '{ print $2 }' | xargs)
do
  ZONE_NET=${zone//.in-addr.arpa./}
  if [[ "$ZONES" != "" ]]; then
    ZONES="$ZONES $ZONE_NET"
  else
    ZONES="$ZONE_NET"
  fi
done

set +e
echo "ZONES: $ZONES"
"""

logger=logging.getLogger(__name__)

def status(format='table'):
    global inline_script
    temp_file='/tmp/ipa_doctor_check.sh'
    temp_out_file='/tmp/ipa_doctor_output.txt'
    with open(temp_file, "w") as f:
        f.write(inline_script)
    os.system("sh %s > %s" % (temp_file, temp_out_file))
    if os.path.exists(temp_out_file):
        map_data={}
        with open(temp_out_file) as file:
            for line in file:
                line_str=line.rstrip()
                if ": " in line_str:
                    splitted=line_str.split(": ")
                    if splitted[1]:
                        map_data[splitted[0]]=splitted[1]
        handle_output(map_data, format)
        
def handle_output(map_data, format):
    ipa_type = map_data["IPA_TYPE"] if "IPA_TYPE" in map_data else None
    zones = map_data["ZONES"].split(" ") if "ZONES" in map_data else list()
    fqdn = map_data["FQDN"] if "FQDN" in map_data else None
    ip = map_data["IP"] if "IP" in map_data else None
    reverse_ip = map_data["REVERSE_IP"] if "REVERSE_IP" in map_data else None
    nslookup_ip = map_data["NSLOOKUP_IP"] if "NSLOOKUP_IP" in map_data else None
    is_reverse_ip_match=False
    zone_map_list=list()
    for zone in zones:
        zone_map={}
        zone_str="%s.in-addr.arpa." % zone
        zone_map["name"]=zone_str
        zone_arr=zone.split(".")
        zone_arr_reversed=reverse(zone_arr)
        if len(zone_arr_reversed) == 1:
            zone_map["match"]="%s.0.0.0/8" % zone_arr_reversed[0]
        if len(zone_arr_reversed) == 2:
            zone_map["match"]="%s.%s.0.0/16" % (zone_arr_reversed[0], zone_arr_reversed[1])
        if len(zone_arr_reversed) == 3:
            zone_map["match"]="%s.%s.%s.0/24" % (zone_arr_reversed[0], zone_arr_reversed[1], zone_arr_reversed[2])
        if reverse_ip.endswith(zone):
            is_reverse_ip_match=True
        zone_map_list.append(zone_map)
    host_details={}
    host_details["type"]=ipa_type
    host_details["ip"]=ip
    host_details["reverse_ip"]=reverse_ip
    host_details["fqdn"]=fqdn
    host_details["reverse_nslookup"]=nslookup_ip
    result_json={}
    result_json['zones']=zone_map_list
    result_json["details"]=host_details
    result_json["dns_zone_match"]="OK" if is_reverse_ip_match else "NOK"
    if format == "json":
        logger.info(response.ResponseHolder(map_content=result_json).to_json(True, True))
    else:
        logger.info("Host details:")
        result_json["details"]["dns_zone_match"]=telemetry_logger.success_message("[OK]") if is_reverse_ip_match else telemetry_logger.fail_message("[OK]")
        hostDetailsTableOut=response.ResponseHolder(map_content=result_json["details"])
        logger.info(hostDetailsTableOut.to_table())
        if result_json["zones"]:
            logger.info("DNS Zones:")
            zonesTableOut=response.ResponseHolder(tuple_content=result_json["zones"])
            logger.info(zonesTableOut.to_table())

def reverse(lst):
    new_lst = lst[::-1]
    return new_lst

if __name__ == "__main__":
    status()
