
from utils import telemetry_logger, response, command_helper
import yaml
import logging
import os
import requests
import sys
from requests.auth import HTTPBasicAuth
from requests.packages.urllib3.exceptions import InsecureRequestWarning

requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

logger=logging.getLogger(__name__)

def status(metrics_str, format, prom_config="/opt/cdp-prometheus/prometheus.yml", request_signer_config="/opt/cdp-request-signer/conf/cdp-request-signer.yaml", remote_write_url=""):
    metrics=[]
    if metrics_str:
         metrics=metrics_str.split(',')
    smon_user=None
    smon_pwd=None
    smon_scheme="http"
    node_exporter_scheme="http"
    node_exporter_user=None
    node_exporter_pwd=None
    all_metrics_strs=[]
    filtered_metrics={}
    if metrics:
        for metric in metrics:
            if format == "json":
                filtered_metrics[metric]="NOK"
            else:
                filtered_metrics[metric]=telemetry_logger.fail_message("[NOK]")
    service_responses={}
    scrapping_tests={}
    json_response={}
    remote_url=get_base_url(remote_write_url) if remote_write_url else None
    remote_proxy_url=None
    remote_no_proxy=None
    
    prometheus_config=None
    if os.path.exists(prom_config):
        with open(prom_config) as file:
            prometheus_config = yaml.load(file, yaml.SafeLoader)
    request_config = None
    if os.path.exists(request_signer_config):
        with open(request_signer_config) as file:
            request_config = yaml.load(file, yaml.SafeLoader)
    request_signer_used=False
    if request_config:
        if not remote_write_url:
            remote_url=get_base_url(request_config["endpoint"])
        if "proxy" in request_config:
            if "https_proxy" in request_config["proxy"] and request_config["proxy"]["https_proxy"]:
                remote_proxy_url=request_config["proxy"]["https_proxy"]
            if "no_proxy" in request_config["proxy"] and request_config["proxy"]["no_proxy"]:
                remote_no_proxy=request_config["proxy"]["no_proxy"]
    request_signer_test_user=None
    request_signer_test_pwd=""
    request_signer_metrics_url=None
    if prometheus_config and "scrape_configs" in prometheus_config and "remote_write" in prometheus_config:
        scrape_cofigs=prometheus_config['scrape_configs']
        node_exporter_job={}
        smon_job={}
        remote_write_urls=prometheus_config['remote_write']
        for remote_write in remote_write_urls:
            if "url" in remote_write:
                if "localhost" in remote_write["url"]:
                    request_signer_used=True
                    request_signer_metrics_url=get_base_url(remote_write["url"]) + "/metrics"
                    if "basic_auth" in remote_write:
                        request_signer_test_user=remote_write["basic_auth"]["username"]
                        if "password_file" in remote_write["basic_auth"]:
                            req_pwd_file=remote_write["basic_auth"]["password_file"]
                            if os.path.exists(req_pwd_file):
                                with open(req_pwd_file, 'r') as file:
                                    request_signer_test_pwd = file.read().replace('\n', '')
                else:
                    remote_url=get_base_url(remote_write["url"])
        for job_conf in scrape_cofigs:
            if job_conf['job_name'] == "node_exporter":
                node_exporter_scheme=job_conf['scheme']
                node_exporter_job=job_conf
                if 'basic_auth' in job_conf:
                    node_exporter_user=job_conf['basic_auth']['username']
                    node_exporter_pwd=job_conf['basic_auth']['password']
            elif job_conf['job_name'] == "smon_health":
                smon_scheme=job_conf['scheme']
                smon_job=job_conf
                if 'basic_auth' in job_conf:
                    smon_user=job_conf['basic_auth']['username']
                    smon_pwd=job_conf['basic_auth']['password']
        if node_exporter_job:
            node_metrics_status=test_metrics_endpoint(node_exporter_job, node_exporter_scheme, node_exporter_user, node_exporter_pwd, all_metrics_strs)
            if format != "json":
                scrapping_tests["Node exporter"]=telemetry_logger.success_message("[OK]") if node_metrics_status == "OK" else telemetry_logger.fail_message("[NOK]")
            else:
                scrapping_tests["node-exporter"]=node_metrics_status
        if smon_job:
            smon_metrics_status=test_metrics_endpoint(smon_job, smon_scheme, smon_user, smon_pwd, all_metrics_strs)
            if format != "json":
                scrapping_tests["SMON exporter"]=telemetry_logger.success_message("[OK]") if smon_metrics_status == "OK" else telemetry_logger.fail_message("[NOK]")
            else:
                scrapping_tests["smon-exporter"]=smon_metrics_status
        if scrapping_tests and format != "json":
            metricsEndpointTestOutputs=response.ResponseHolder(map_content=scrapping_tests)
            logger.info("Scrapping tests:")
            logger.info(metricsEndpointTestOutputs.to_table())
        if format == "json":
            json_response["scrapping"]=scrapping_tests
        if all_metrics_strs:
            for metrics_str in all_metrics_strs:
                for metric_name in metrics:
                    if metrics_str.startswith(metric_name):
                        if format == "json":
                            filtered_metrics[metric_name]="OK"
                        else:
                            filtered_metrics[metric_name]=telemetry_logger.success_message("[OK]")
        if format == "json":
            json_response["metrics"]=filtered_metrics
        elif filtered_metrics:
            metricsTestOutputs=response.ResponseHolder(map_content=filtered_metrics)
            logger.info("Metric patterns found:")
            logger.info(metricsTestOutputs.to_table())
        services_to_check = ["cdp-prometheus", "cdp-node-exporter", "cdp-blackbox-exporter"]
        if request_signer_used:
            services_to_check.append("cdp-request-signer")
        for service in services_to_check:
            _,_, ret = command_helper.run_command("systemctl is-active --quiet %s" % service , True)
            if format != "json":
                service_responses[service]=telemetry_logger.success_message("[OK]") if ret == 0 else telemetry_logger.fail_message("[NOK]")
            else:
                service_responses[service]="OK" if ret == 0 else "NOK"
        if format != "json":
            serviceTableOutput=response.ResponseHolder(map_content=service_responses)
            logger.info("Service states:")
            logger.info(serviceTableOutput.to_table())
        else:
            json_response["services"]=service_responses
    else:
        if format != "json": 
            logger.error("Cannot find monitoring configuration. Exiting ...")
            sys.exit(1)
    
    if remote_url:
        status, code=remote_metrics_health_check(remote_url, remote_proxy_url, remote_no_proxy)
        endpoint_check={}
        endpoint_check["url"]=remote_url
        endpoint_check["proxy"]="Yes" if remote_proxy_url else "No"
        endpoint_check["code"]=code
        if format == "json":
            endpoint_check["status"]=status
            json_response["endpoint"]=endpoint_check
        else:
            endpoint_check["status"]=telemetry_logger.success_message("[OK]") if status == "OK" else telemetry_logger.fail_message("[NOK]")
            endpointCheckTableOut=response.ResponseHolder(map_content=endpoint_check)
            logger.info("Endpoint details:")
            logger.info(endpointCheckTableOut.to_table())
    if request_signer_metrics_url and "cdp-request-signer" in service_responses:
        request_signer={}
        succ_count=-1
        err_count=-1
        status="NOK"
        if "NOK" in service_responses["cdp-request-signer"]:
            request_signer["status"]="NOK" if format == "json" else telemetry_logger.fail_message("[NOK]")
        else:
            status, succ_count, err_count = get_request_signer_metrics(request_signer_metrics_url, request_signer_test_user, request_signer_test_pwd)
            request_signer["success_count"]=succ_count
            request_signer["error_count"]=err_count
        if format == "json":
            request_signer["status"]=status
            json_response["request-signer"]=request_signer
        else:
            request_signer["status"]=telemetry_logger.success_message("[OK]") if status == "OK" else telemetry_logger.fail_message("[NOK]")
            request_signer_tuple_list=list()
            if succ_count != -1:
                request_signer_tuple_list.append(("Successful", request_signer["success_count"]))
            if succ_count != -1:
                request_signer_tuple_list.append(("Failed", request_signer["error_count"]))
            request_signer_tuple_list.append(("Status", request_signer["status"]))
            requestSignerTableOut=response.ResponseHolder(tuple_content=request_signer_tuple_list)
            logger.info("Request signer details:")
            logger.info(requestSignerTableOut.to_table())
    
    if format == "json":
        logger.info(response.ResponseHolder(map_content=json_response).to_json(True, True))

def test_metrics_endpoint(job, scheme, user, pwd, all_metrics_strs):
    static_conf=job["static_configs"][0]
    targets=static_conf["targets"]
    target_url=targets[0] if type(targets) is list else targets
    url="%s://localhost:%s/metrics" % (scheme, target_url.split(":")[1])
    status="NOK"
    try:
        resp=requests.get(url, auth=HTTPBasicAuth(user, pwd), timeout=10, verify=False, stream=True)
        if resp.status_code == 200:
            number_of_metrics=0
            for line in resp.iter_lines():
                line_str=line.decode()
                if not line_str.startswith("#"):
                    number_of_metrics+=1
                    all_metrics_strs.append(line_str)
            if number_of_metrics > 5:
                status = "OK"
    except requests.exceptions.ConnectionError:
        pass
    return status

def get_base_url(url):
    split_by_scheme=url.split("://")
    without_scheme=split_by_scheme[0] if len(split_by_scheme) == 1 else split_by_scheme[1]
    split_by_slash=without_scheme.split("/")[0]
    return "https://%s" % split_by_slash if len(split_by_scheme) == 1 else "%s://%s" % (split_by_scheme[0], split_by_slash)

def get_request_signer_metrics(request_signer_url, request_signer_user, request_signer_pwd):
    status="NOK"
    success_count=0
    error_count=0
    try:
        if request_signer_user:
            r = requests.get(request_signer_url, auth=HTTPBasicAuth(request_signer_user, request_signer_pwd), timeout=10, verify=False, stream=True)
            response_code=r.status_code
        else:
            r = requests.get(request_signer_url, timeout=10, verify=False, stream=True)
            response_code=r.status_code
        if response_code > 199 and response_code < 499 and response_code != 403:
            status="OK"
            for line in r.iter_lines():
                line_str=line.decode()
                if "status_code=\"2" in line_str or "status_code=\"3" in line_str:
                    spitted_line=line_str.split("} ")
                    if len(spitted_line) > 1 and spitted_line[1].isnumeric():
                        success_count+=int(spitted_line[1])
                if "status_code=\"4" in line_str or "status_code=\"5" in line_str:
                    spitted_line=line_str.split("} ")
                    if len(spitted_line) > 1 and spitted_line[1].isnumeric():
                        error_count+=int(spitted_line[1])
            return status, success_count, error_count
    except requests.exceptions.RequestException:
        pass
    except requests.exceptions.HTTPError:
        pass
    except requests.exceptions.ConnectionError:
        pass
    except requests.exceptions.Timeout:
        pass
    return status, -1, -1

def remote_metrics_health_check(remote_url, remote_proxy_url, remote_no_proxy):
    status="NOK"
    response_code="UNKNOWN"
    if remote_no_proxy:
        os.environ["NO_PROXY"]=remote_no_proxy
        os.environ["no_proxy"]=remote_no_proxy
    if remote_proxy_url:
        os.environ["HTTPS_PROXY"]=remote_proxy_url
        os.environ["https_proxy"]=remote_proxy_url
    try:
        r = requests.get(remote_url,timeout=10)
        response_code=r.status_code
        if response_code > 199 and response_code < 499 and response_code != 403:
            status="OK"
    except requests.exceptions.Timeout as errt:
        response_code = response_code + " (Timeout)"
    except requests.exceptions.ConnectionError as errc:
        response_code = response_code + " (Connection Error)"
    except requests.exceptions.HTTPError as errh:
        response_code = response_code + " (Http Error)"
    except requests.exceptions.RequestException as err:
        response_code = response_code + " (Request Error)"
    if remote_proxy_url:
        os.environ["HTTPS_PROXY"]=""
        os.environ["https_proxy"]=""
    if remote_no_proxy:
        os.environ["NO_PROXY"]=""
        os.environ["no_proxy"]=""
    return status, response_code