#!/usr/bin/env python

from utils import ccm_config_holder, logging_agent_config_holder, proxy_settings, connection, telemetry_logger, response, aws_ec2_metadata, version, doctor_output
from netifaces import AF_INET
from google.protobuf.json_format import MessageToDict
from nodestatus.generated import nodestatus_pb2 as NodeStatus

import datetime
import socks
import socket
import dns.resolver
import ipaddress
import netifaces
import re
import logging
import requests
import os
import yaml

logger=logging.getLogger(__name__)

def main(output_format='table', scan_neighbours=False, skip_scan_neighbours=True, include_neighbours=False, databus_address='https://dbusapi.us-west-1.altus.cloudera.com', databus_s3_address=None, cloud_provider=None, pretty=False):
    return network_status(output_format, scan_neighbours, skip_scan_neighbours, include_neighbours, databus_address, databus_s3_address, cloud_provider, pretty)

def network_status(output_format='table', scan_neighbours=False, skip_scan_neighbours=True, include_neighbours=False, databus_address='https://dbusapi.us-west-1.altus.cloudera.com', databus_s3_address=None, cloud_provider=None, pretty=False, silenced=False, skip_ccm=False):
    
    if skip_scan_neighbours is True:
        logger.warn("deprecated skip_scan_neighbours flag is used but it's going to be ignored since it's deprecated")
    hostname = socket.gethostname()
    binaryVersion=version.get_binary_version()
    networkDetails=NodeStatus.NetworkDetails(cdpTelemetryVersion=binaryVersion)
    networkDetails.host=hostname

    colored_hostname = telemetry_logger.header_message(hostname)
    if is_not_json_format(output_format):
        logger.info("[START network checks for node {0}]\n".format(colored_hostname))
    else:
        logger.debug("[START network checks for node {0}]\n".format(colored_hostname))

    ccmConfigHolder=ccm_config_holder.CcmConfigHolder()
    ccmEnabled=ccmConfigHolder.is_ccm_enabled()
    ccm_host, ccm_port = '', ''
    ccm_proxy_host, ccm_proxy_port, ccm_proxy_user, ccm_proxy_password = None, None, None, None
    ccm_proxy_url=None
    ccm_v2=True
    ccmV2Url=None
    ccmV2ProxySettings=None
    if ccmEnabled and not skip_ccm:
        ccm_v2=ccmConfigHolder.is_v2()
        ccm_host, ccm_port = getCCMConfig(ccmConfigHolder, output_format)
        if ccm_v2:
            ccmV2Url=ccmConfigHolder.get_ccm_v2_url()
            ccmV2ProxySettings = ccmConfigHolder.get_ccm_v2_proxy_settings()
            if ccmV2ProxySettings:
                ccm_proxy_host, ccm_proxy_port = ccmV2ProxySettings.get_host_and_port()
                ccm_proxy_user = ccmV2ProxySettings.get_user()
                ccm_proxy_password = ccmV2ProxySettings.get_password()
                ccm_proxy_url = ccmV2ProxySettings.get_url()
        else:
            ccm_proxy_host, ccm_proxy_port, ccm_proxy_user, ccm_proxy_password = ccmConfigHolder.get_ccm_v1_proxy_settings()
    logProxySettings(ccm_proxy_host, ccm_proxy_port, "CCM", output_format)

    loggingAgentConfigHolder=logging_agent_config_holder.LoggingAgentConfigHolder()

    proxy_host = ccm_proxy_host if ccm_proxy_host else ""
    proxy_port = ccm_proxy_port if ccm_proxy_port else ""
    proxy_user = ccm_proxy_user if ccm_proxy_user else ""
    proxy_password = ccm_proxy_password if ccm_proxy_password else ""
    proxy_url = ccm_proxy_url if "" else ccm_proxy_url
    if not proxy_url and ccm_proxy_host:
        if ccm_proxy_port:
            if ccm_proxy_port == "443":
                proxy_url="https://%s:%s" % (ccm_proxy_host, ccm_proxy_port)
            else:
                proxy_url="http://%s:%s" % (ccm_proxy_host, ccm_proxy_port)
        else:
            proxy_url="http://%s:80" % ccm_proxy_host
    loggingProxyConfigs=loggingAgentConfigHolder.get_proxy_configs()
    if loggingProxyConfigs:
        proxy_host, proxy_port = loggingProxyConfigs.get_host_and_port()
        logProxySettings(proxy_host, proxy_port, "Logging Agent", output_format)
        proxy_url = loggingProxyConfigs.get_url()
        proxy_user = loggingProxyConfigs.get_user()
        proxy_password = loggingProxyConfigs.get_password()
    skipDatabusRelatedProxy=True if ccm_proxy_url and loggingAgentConfigHolder.is_any_databus_config() and not loggingProxyConfigs else False
    cloudera_com_is_accessible = False
    if proxy_url:
        cloudera_com_is_accessible = http_test("https://cloudera.com", proxy_url=proxy_url, proxy_user=proxy_user, proxy_password=proxy_password, output_format=output_format)
    else:
        cloudera_com_is_accessible = socket_test(host="cloudera.com", port=80, timeout=3, proxy_host=proxy_host, proxy_port=proxy_port, proxy_user=proxy_user, proxy_password=proxy_password)
    if loggingAgentConfigHolder.is_any_databus_config():
        collected_databus_address=loggingAgentConfigHolder.get_databus_endpoint()
        if collected_databus_address:
            databus_address=collected_databus_address
    monitoring_endpoint=get_metrics_endpoint()
    dbus_https=True if databus_address.startswith("https://") else False
    dbus_host=databus_address.replace("https://","").replace("http://", "")
    dbus_port=443 if dbus_https else 80
    dbus_api_is_accessible=http_test(databus_address, proxy_url, proxy_user, proxy_password, output_format=output_format, skipProxy=skipDatabusRelatedProxy)

    dbus_api_s3_pair_map={}
    dbus_api_s3_pair_map["dbusapi.sigma-dev"]="https://cloudera-dbus-dev.s3.amazonaws.com"
    dbus_api_s3_pair_map["dbusapi.sigma-stage"]="https://cloudera-dbus-stage.s3.amazonaws.com"
    dbus_api_s3_pair_map["dbusapi.us-west-1"]="https://cloudera-dbus-prod.s3.amazonaws.com"

    if databus_s3_address:
        dbus_s3_url=databus_s3_address
    else:
        dbus_s3_url=None
        for k, v in dbus_api_s3_pair_map.items():
            if k in dbus_host:
                dbus_s3_url=v
                break
    dbus_s3_url_is_accessible = False
    if dbus_s3_url:
        dbus_s3_url_is_accessible = http_test(dbus_s3_url, proxy_url=proxy_url, proxy_user=proxy_user, proxy_password=proxy_password, output_format=output_format, skipProxy=skipDatabusRelatedProxy)
    dns_resolver = dns.resolver.Resolver()
    nameservers=dns_resolver.nameservers
    clouderaComAccessible=getResult(cloudera_com_is_accessible, output_format)
    results = [
        ["DNS resolvers", str(nameservers)],
        ["cloudera.com is accessible", clouderaComAccessible]
    ]
    
    if monitoring_endpoint:
        monitoring_endpoint_is_accessible=http_test(monitoring_endpoint, proxy_url=proxy_url, proxy_user=proxy_user, proxy_password=proxy_password, output_format=output_format)
        monitoringEndpointAccessible=getResult(monitoring_endpoint_is_accessible, output_format)
        results.append(["Compute monitoring API URL", monitoring_endpoint])
        results.append(["Compute monitoring API is accessible", monitoringEndpointAccessible])
    archive_cloudera_com_is_accessible=http_test("https://archive.cloudera.com", proxy_url=proxy_url, proxy_user=proxy_user, proxy_password=proxy_password, output_format=output_format)
    archiveClouderaComAccessible=getResult(archive_cloudera_com_is_accessible, output_format)
    service_delivery_cache_s3_is_accessible=http_test("https://cloudera-service-delivery-cache.s3.amazonaws.com", proxy_url=proxy_url, proxy_user=proxy_user, proxy_password=proxy_password, output_format=output_format)
    serviceDeliveryCacheS3Accessible=getResult(service_delivery_cache_s3_is_accessible, output_format)
    results.append(["archive.cloudera.com is accessible", archiveClouderaComAccessible])
    results.append(["Service Delivery S3 Cache is accessible", serviceDeliveryCacheS3Accessible])

    networkDetails.dnsResolvers.extend(nameservers)
    if not is_not_json_format(output_format):
        networkDetails.clouderaComAccessible=NodeStatus.HealthStatus.Value(clouderaComAccessible)
        networkDetails.archiveClouderaComAccessible=NodeStatus.HealthStatus.Value(archiveClouderaComAccessible)
        networkDetails.serviceDeliveryCacheS3Accessible=NodeStatus.HealthStatus.Value(serviceDeliveryCacheS3Accessible)
        if monitoring_endpoint:
            networkDetails.computeMonitoringAccessible=NodeStatus.HealthStatus.Value(archiveClouderaComAccessible)
    if databus_address:
        databusAccessible=getResult(dbus_api_is_accessible, output_format)
        results.append(["DataBus API address", databus_address])
        results.append(["DataBus API is accessible", databusAccessible])
        if not is_not_json_format(output_format):
            networkDetails.databusAccessible=NodeStatus.HealthStatus.Value(databusAccessible)
        if dbus_s3_url:
            databusS3Accessible=getResult(dbus_s3_url_is_accessible, output_format)
            results.append(["DataBus S3 endpoint", dbus_s3_url])
            results.append(["DataBus S3 URL is accessible", databusS3Accessible])
            if not is_not_json_format(output_format):
                networkDetails.databusS3Accessible=NodeStatus.HealthStatus.Value(databusS3Accessible)
    
    if cloud_provider:
        if cloud_provider.upper() == "AWS":
            logging_output_conf=loggingAgentConfigHolder.get_output_configs()
            aws_region=None
            if "s3_region" in logging_output_conf:
                aws_region=logging_output_conf["s3_region"]
            else:
                aws_region=aws_ec2_metadata.get_aws_instance_region()
            aws_domain=aws_ec2_metadata.get_aws_instance_domain()
            if aws_domain is None:
                aws_domain="amazonaws.com"
            url_suffix="amazonaws.com"
            if aws_region and aws_domain:
                url_suffix="%s.%s" % (aws_region, aws_domain)
            s3_endpoint="s3.%s" % url_suffix
            s3_endpoint_accessible=socket_test(host=s3_endpoint, port=443, timeout=3)
            s3EndpointAccessible=getResult(s3_endpoint_accessible, output_format)
            results.append(["S3 endpoint", s3_endpoint])
            results.append(["S3 is accessible", s3EndpointAccessible])
            if not is_not_json_format(output_format):
                networkDetails.s3Accessible=NodeStatus.HealthStatus.Value(s3EndpointAccessible)
            sts_endpoint="sts.%s" % url_suffix
            sts_endpoint_accessible=socket_test(host=sts_endpoint, port=443, timeout=3)
            stsEndpointAccessible=getResult(sts_endpoint_accessible, output_format)
            results.append(["STS endpoint", sts_endpoint])
            results.append(["STS is accessible", stsEndpointAccessible])
            if not is_not_json_format(output_format):
                networkDetails.stsAccessible=NodeStatus.HealthStatus.Value(stsEndpointAccessible)
        if cloud_provider.upper() == "AZURE":
            azure_mgmt_endpoint="management.azure.com"
            azure_mgmt_accessible=socket_test(host=azure_mgmt_endpoint, port=443, timeout=3)
            azureMgmtEndpointAccessible=getResult(azure_mgmt_accessible, output_format)
            results.append(["Azure mgmt endpoint", azure_mgmt_endpoint])
            results.append(["Azure mgmt is accessible", azureMgmtEndpointAccessible])
            if not is_not_json_format(output_format):
                networkDetails.azureManagementAccessible=NodeStatus.HealthStatus.Value(azureMgmtEndpointAccessible)
            logging_output_conf=loggingAgentConfigHolder.get_output_configs()
            if logging_output_conf and "azure_storage_account" in logging_output_conf:
                azure_storage_account=logging_output_conf["azure_storage_account"]
                azure_storage_host="%s.dfs.core.windows.net" % azure_storage_account
                azure_storage_port=443
                azure_storage_is_accessible = socket_test(host=azure_storage_host, port=azure_storage_port, timeout=3)
                azureStorageEndpointAccessible=getResult(azure_storage_is_accessible, output_format)
                results.append(["ADLSv2 endpoint", azure_storage_host])
                results.append(["ADLSv2 is accessible", azureStorageEndpointAccessible])
                if not is_not_json_format(output_format):
                    networkDetails.adlsV2Accessible=NodeStatus.HealthStatus.Value(azureStorageEndpointAccessible)
        if cloud_provider.upper() == "GCP":
            gcs_endpoint="storage.googleapis.com"
            gcs_accessible=socket_test(host=gcs_endpoint, port=443, timeout=3)
            gcsEndpointAccessible=getResult(gcs_accessible, output_format)
            results.append(["GCS endpoint", gcs_endpoint])
            results.append(["GCS is accessible", gcsEndpointAccessible])
            if not is_not_json_format(output_format):
                networkDetails.gcsAccessible=NodeStatus.HealthStatus.Value(gcsEndpointAccessible)
    if ccm_host and ccm_port:
        if ccm_v2:
            cacert, cert, key = ccmConfigHolder.get_ccm_v2_certs()
            ccmAccessible=getResult(http_test(ccmV2Url, ccm_proxy_url, ccm_proxy_user, ccm_proxy_password, output_format, cacert=cacert, cert=cert, key=key), output_format)
            ccmConfigHolder.cleanup_jumpgate_tmp_certs()
        else:
            ccmAccessible=getResult(is_ccm_accessible(ccm_host, ccm_port, proxy_host=ccm_proxy_host, proxy_port=ccm_proxy_port, proxy_user=ccm_proxy_user, proxy_password=ccm_proxy_password), output_format)
        results.append(["CCM address", ccm_host + ":" + ccm_port])
        results.append(["DNS Answers for CCM", get_ccm_answers(ccm_host, output_format)])
        results.append(["CCM is accessible", ccmAccessible])
        #networkDetails.ccmAddress=ccm_host + ":" + ccm_port
        networkDetails.ccmEnabled=True
        if not is_not_json_format(output_format):
            networkDetails.ccmAccessible=NodeStatus.HealthStatus.Value(ccmAccessible)
    else:
        results.append(["CCM", "disabled"])
        networkDetails.ccmEnabled=False
        if not is_not_json_format(output_format):
            networkDetails.ccmAccessible=NodeStatus.HealthStatus.UNKNOWN

    if is_not_json_format(output_format):
        logger.info(telemetry_logger.format_table(results, headers=[]))
    if not is_not_json_format(output_format): 
        networkDetails.anyNeighboursAccessible=NodeStatus.HealthStatus.UNKNOWN
        networkDetails.neighbourScan=False
    if scan_neighbours:
        networkDetails.neighbourScan=True
        result, neighbours = check_neighbours(output_format)
        if result:
            if is_not_json_format(output_format):
                logger.info(telemetry_logger.success_message("Saltboot API could be reached on at least one of the neighbours."))
            else:
                networkDetails.anyNeighboursAccessible=NodeStatus.HealthStatus.OK
                if include_neighbours:
                    networkDetails.neighbours.extend(neighbours)
                else:
                    networkDetails.numberOfNeighbours=len(neighbours)
        else:
            if is_not_json_format(output_format):
                logger.error(telemetry_logger.fail_message("Error: Saltboot API could NOT be reached on any of the neighbours:" + str(neighbours)))
            else:
                if not is_not_json_format(output_format):
                    networkDetails.anyNeighboursAccessible=NodeStatus.HealthStatus.NOK
                if include_neighbours:
                    networkDetails.neighbours.extend(neighbours)
                else:
                    networkDetails.numberOfNeighbours=len(neighbours)
    if not is_not_json_format(output_format):
        nowTs=int(datetime.datetime.now().strftime('%s'))
        networkDetails.timestamp=nowTs
        #networkDetailsProtoDict = MessageToDict(networkDetails, including_default_value_fields=True)
        networkDetailsProtoDict = MessageToDict(networkDetails)
        if not silenced:
            networkJsonOutput=response.ResponseHolder(map_content=networkDetailsProtoDict).to_json(pretty=pretty, sortKeys=True)
            logger.info(networkJsonOutput)
            doctor_output.save_doctor_status(networkJsonOutput, "network")
        return networkDetailsProtoDict

def get_addresses(output_format='table', databus_address='https://dbusapi.us-west-1.altus.cloudera.com', databus_s3_address=None, cloud_provider=None, pretty=False):
    result={}
    result["cloud"]=list()
    result["cloudera"]=list()
    result["cloudera"].append("https://archive.cloudera.com")
    result["cloudera"].append("https://cloudera-service-delivery-cache.s3.amazonaws.com")
    loggingAgentConfigHolder=logging_agent_config_holder.LoggingAgentConfigHolder()
    outputConf=loggingAgentConfigHolder.get_output_configs()
    if "s3_bucket" in outputConf:
        cloud_provider="AWS"
    if "azure_storage_account" in outputConf:
        cloud_provider="AZURE"
    if "blind_write" in outputConf:
        cloud_provider="GCS"
    if loggingAgentConfigHolder.is_any_databus_config():
        collected_databus_address=loggingAgentConfigHolder.get_databus_endpoint()
        if collected_databus_address:
            databus_address=collected_databus_address
    result["cloudera"].append(databus_address)
    cloud_provider=cloud_provider.upper()
    if cloud_provider == "AWS":
        aws_region=None
        if "s3_region" in outputConf:
            aws_region=outputConf["s3_region"]
        else:
            aws_region=aws_ec2_metadata.get_aws_instance_region()
        aws_domain=aws_ec2_metadata.get_aws_instance_domain()
        if aws_domain is None:
            aws_domain="amazonaws.com"
        url_suffix="amazonaws.com"
        if aws_region and aws_domain:
            url_suffix="%s.%s" % (aws_region, aws_domain)
        s3_endpoint="https://s3.%s" % url_suffix
        sts_endpoint="https://sts.%s" % url_suffix
        result["cloud"].append(s3_endpoint)
        result["cloud"].append(sts_endpoint)
    elif cloud_provider == "AZURE":
        azure_storage_account=outputConf["azure_storage_account"]
        azure_storage_host="https://%s.dfs.core.windows.net" % azure_storage_account
        azure_mgmt_endpoint="https://management.azure.com"
        result["cloud"].append(azure_storage_host)
        result["cloud"].append(azure_mgmt_endpoint)
    elif cloud_provider == "GCP":
        result["cloud"].append("https://storage.googleapis.com")
    if databus_s3_address:
        result["cloudera"].append(databus_s3_address)
    ccmConfigHolder=ccm_config_holder.CcmConfigHolder()
    ccmEnabled=ccmConfigHolder.is_ccm_enabled()
    if ccmEnabled and ccmConfigHolder.is_v2():
        result["cloudera"].append(ccmConfigHolder.get_ccm_v2_url())
    if is_not_json_format(output_format):
        logger.info("Cloud Addresses:")
        for c in result["cloud"]:
            logger.info("- %s" % c)
        logger.info("Cloudera Addresses:")
        for c in result["cloudera"]:
            logger.info("- %s" % c)
    else:
        addressesJsonOutput=response.ResponseHolder(map_content=result).to_json(pretty=pretty, sortKeys=True)
        logger.info(addressesJsonOutput)

def getResult(result, output_format='table'):
    if is_not_json_format(output_format):
        if result:
            return telemetry_logger.success_message(str(result))
        else:
            return telemetry_logger.fail_message(str(result))
    else:
        return "OK" if result else "NOK"

def get_ccm_answers(ccm_host, output_format):
    try:
        answers = dns.resolver.resolve(ccm_host, 'A')
        addresses = []
        for answer in answers:
            addresses.append(answer.address)
        return addresses
    except Exception as e:
        logger.debug(str(e))
        if is_not_json_format(output_format):
            return telemetry_logger.fail_message("Failed")
        else:
            return "NOK"

def is_ccm_accessible(ccm_host, ccm_port, proxy_host='', proxy_port='', proxy_user='', proxy_password=''):
    return socket_test(host=ccm_host, port=int(ccm_port), timeout=3, proxy_host=proxy_host, proxy_port=proxy_port, proxy_user=proxy_user, proxy_password=proxy_password)

def getCCMConfig(ccmConfigHolder, output_format='table'):
    ccm_host = ""
    ccm_port = ""
    try:
        ccm_host, ccm_port = ccmConfigHolder.get_host_and_port()
        if ccm_host:
            return ccm_host, ccm_port
        else:
            if is_not_json_format(output_format):
                logger.error(telemetry_logger.fail_message("Could not find " + ccmConfigHolder.get_ccm_v1_config_filename() + " or " + ccmConfigHolder.get_ccm_v2_config_filename() + " or " + ccmConfigHolder.get_ccm_v2_jumpgate_config_filename()))
            else:
                logger.debug(telemetry_logger.fail_message("Could not find " + ccmConfigHolder.get_ccm_v1_config_filename() + " or " + ccmConfigHolder.get_ccm_v2_config_filename() + " or " + ccmConfigHolder.get_ccm_v2_jumpgate_config_filename())) 
    except PermissionError:
        logger.error(telemetry_logger.fail_message("Please run this command as superuser!"))
        exit()
    return ccm_host, ccm_port

def logProxySettings(proxy_host, proxy_port, prefix, output_format='table'):
    if (proxy_host):
        if is_not_json_format(output_format):
            logger.info(telemetry_logger.bold_message(prefix + " proxy settings found: Host: " + proxy_host + ", Port: " + proxy_port))
        else:
            logger.debug(telemetry_logger.bold_message(prefix + "Proxy settings found: Host: " + proxy_host + ", Port: " + proxy_port))

def http_test(address, proxy_url='', proxy_user='', proxy_password='', output_format = 'table', skipProxy=False, cacert=None, cert=None, key=None):
    clearProxy=False
    proxySettings=None
    if not skipProxy and proxy_url:
        if is_not_json_format(output_format):
            if requests.utils.should_bypass_proxies(address, None):
                logger.info("Bypass proxies for %s" % address)
            else:
                logger.info("Using %s proxy url for %s" % (proxy_url, address))
        else:
            if requests.utils.should_bypass_proxies(address, None):
                logger.debug("Bypass proxies for %s" % address)
            else:
                logger.debug("Using %s proxy url for %s" % (proxy_url, address))
        proxySettings=proxy_settings.ProxySettings(proxy_url=proxy_url, proxy_user=proxy_user, proxy_password=proxy_password)
    else:
        clearProxy=True
        if skipProxy:
            if is_not_json_format(output_format):
                logger.info("Skipping proxy usage for %s" % address)
            else:
                logger.debug("Skipping proxy usage for %s" % address) 
    return connection.connected(address, proxySettings=proxySettings, clearProxy=clearProxy, cacert=cacert, cert_key_pair=(cert, key))

def socket_test(host, port, timeout, proxy_host='', proxy_port='', proxy_user='', proxy_password=''):
    try:
        s = socks.socksocket()
        if proxy_host and proxy_port:
            s.set_proxy(socks.HTTP, proxy_host, port=int(proxy_port), username=proxy_user, password=proxy_password)
        s.settimeout(timeout)
        s.connect((host, int(port)))
        return True
    except socket.error as ex:
        logger.debug(str(ex))
        return False

def check_neighbours(output_format='table'):
    from scapy.all import conf,get_if_list
    interfaces = get_if_list()
    if is_not_json_format(output_format):
        logger.info("Found interfaces:" + str(interfaces))
    else:
        logger.debug("Found interfaces:" + str(interfaces))
    domain_name = get_domain_name()
    if is_not_json_format(output_format):
        logger.info("Domain name of the machine: " + domain_name)
    else:
        logger.debug("Domain name of the machine: " + domain_name)
    scanned_networks = []
    found_neighbours = []

    for interface in interfaces:
        addresses = netifaces.ifaddresses(conf.iface.name)
        if is_not_json_format(output_format):
            logger.info("Scanning network interface: "+ interface)
        else:
            logger.debug("Scanning network interface: "+ interface)
        if AF_INET in addresses:
          for addr in netifaces.ifaddresses(conf.iface.name)[AF_INET]:
            logger.debug("Found address:" + addr["addr"])
            ipaddr = ipaddress.ip_interface(addr["addr"]+"/"+addr["netmask"])
            ipnetwork = str(ipaddr.network)
            if ipnetwork not in scanned_networks:
                logger.debug("Found network to be scanned:" + ipnetwork)
                found_neighbours.extend(get_neighbours_with_arp(ipnetwork))
                scanned_networks.append(ipnetwork)
            else:
                logger.debug(ipnetwork + " has already been scanned.")

    any_neighbour_reached = False
    found_neighbours = slice_neighbour_count(found_neighbours)

    for ipv4 in found_neighbours:
        socket.setdefaulttimeout(1)
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        result = sock.connect_ex((ipv4,7070))
        if result == 0:
           logger.debug("Looking for saltboot on port 7070, could be reached on: " + ipv4)
           any_neighbour_reached = True
        else:
            logger.debug("Looking for saltboot on port 7070, could NOT be reached on: " + ipv4)
        sock.close()

    return any_neighbour_reached, found_neighbours

def slice_neighbour_count(neighbours):
    max_neighbour_count = 40
    actual_neighbour_count = len(neighbours)
    if actual_neighbour_count > max_neighbour_count:
        logger.warn("Found too many neightbours: %d, reducing neighbour number to  maximum allowed: %d ", actual_neighbour_count, max_neighbour_count)
    return neighbours[:max_neighbour_count]

def get_neighbours_with_arp(cidr):
    from scapy.all import srp,Ether,ARP,conf
    logger.debug("Scanning network(%s) with ARP", cidr)
    neighbours_ipv4 = []
    conf.verb=0
    ans,uans=srp(Ether(dst="FF:FF:FF:FF:FF:FF")/ARP(pdst=cidr),timeout=1)
    for snd,rcv in ans:
            logger.debug(rcv.sprintf(r"Found machine with MAC:%Ether.src% - IP:%ARP.psrc%"))
            ipv4 = rcv.sprintf(r"%ARP.psrc%")
            neighbours_ipv4.append(ipv4)
    return neighbours_ipv4

def get_domain_name():
    fqdn = socket.getfqdn(socket.getfqdn())
    splitFqdn = fqdn.split('.', 1)
    if (len(splitFqdn) > 1):
        return splitFqdn[1]
    else:
        return ""

def is_not_json_format(output_format):
    return not output_format or not output_format == "json"

def get_metrics_endpoint():
    #request_signer_config="examples/monitoring/cdp-request-signer.yaml"
    request_signer_config="/opt/cdp-request-signer/conf/cdp-request-signer.yaml"
    result = None
    request_config = None
    if os.path.exists(request_signer_config):
        with open(request_signer_config) as file:
            request_config = yaml.load(file, yaml.SafeLoader)
        if request_config:
            return get_base_url(request_config["endpoint"])
    return result

def get_base_url(url):
    # todo: same as in monitoring - move this to utils
    split_by_scheme=url.split("://")
    without_scheme=split_by_scheme[0] if len(split_by_scheme) == 1 else split_by_scheme[1]
    split_by_slash=without_scheme.split("/")[0]
    return "https://%s" % split_by_slash if len(split_by_scheme) == 1 else "%s://%s" % (split_by_scheme[0], split_by_slash)

if __name__ == '__main__':
   main()