#!/usr/bin/env python
import sys

import requests
from tabulate import tabulate
import socket
import logging
from utils import telemetry_logger

logger=logging.getLogger(__name__)

def run(imageId):
    hostname = socket.gethostname()
    colored_hostname = telemetry_logger.header_message(hostname)
    logger.info("[START Validating the accessibility of the parcels on node {0}]\n".format(colored_hostname))

    if not imageId:
        imageId = '6fc6aa99-36ef-48d4-59bd-abe236530219'
    data = download_json('https://cloudbreak-imagecatalog.s3.amazonaws.com/v3-rc-cb-image-catalog.json')
    images = data['images']['cdh-images']

    cdh_parcel_url = []
    cm_parcel_url = []
    pre_warm_parcel_url = []
    pre_warm_csd_url = []

    for i in images:
        if i['uuid'] == imageId:
            os_type = i['os_type']
            cdh_parcel_url.append(i['stack-details']['repo']['stack'][os_type])
            cm_parcel_url.append(i['repo'][os_type])
            pre_warm_parcel_url = get_pre_warm_parcel_urls(i['pre_warm_parcels'])
            pre_warm_csd_url = i['pre_warm_csd']

    if not cdh_parcel_url and not cm_parcel_url and not pre_warm_parcel_url and not pre_warm_csd_url:
        logger.info("There are no parcels fount on the current image.")
    else:
        verify_parcels("CDH", cdh_parcel_url)
        verify_parcels("CM", cm_parcel_url)
        verify_parcels("Pre warm", pre_warm_parcel_url)
        verify_parcels("CSD", pre_warm_csd_url)

    logger.info("[END parcel checks]")


def get_pre_warm_parcel_urls(pre_warm_parcels):
    pre_warm_parcel_urls = []
    for i in pre_warm_parcels:
        pre_warm_parcel_urls.append(i[1])

    return pre_warm_parcel_urls


def verify_parcels(parcel_type, parcels):
    if parcels:
        results = []
        for parcelUrl in parcels:
            result_pair = [parcelUrl, get_parcel_report_by_name(parcelUrl)]
            results.append(result_pair)
        print_table_format(parcel_type, results)


def get_parcel_report_by_name(url):
    if check_connection(url) != 404:
        return telemetry_logger.success_message("[OK]")
    else:
        return telemetry_logger.fail_message("[NOK]")


def check_connection(url):
    try:
        page = requests.get(url)
        return page.status_code
    except Exception:
        return 500


def download_json(url):
    try:
        response = requests.get(url)
        return response.json()
    except Exception:
        logger.info("Failed to download image catalog. Please make sure that you can reach this URL: %s", url)
        logger.info("[END parcel checks]")
        sys.exit(1)

def print_table_format(parcel_type, results):
    logger.info(tabulate(results, headers=[parcel_type + ' parcel URL', 'Status'], tablefmt='pretty'))
