#!/usr/bin/env python

import os
import re
import shutil
import subprocess
import socket
import logging

from utils import telemetry_logger
from glob import glob

logger=logging.getLogger(__name__)

def fetch_results():
    hostname = socket.gethostname()
    colored_hostname=telemetry_logger.header_message(hostname)
    logger.info("[START recipe checks for node {0}]\n".format(colored_hostname))

    executedRecipes = glob("/var/log/recipes/*")

    recipeTypes = []

    for path in executedRecipes:
        recipeTypes.append(os.path.basename(path))

    logger.info("Recipe hook directories: {}".format(recipeTypes))

    tableData = []

    for path in executedRecipes:
        # print(path)
        rowData = []
        for hookPath in glob(path + "/*"):
            if ".log" in os.path.basename(hookPath):
                rowData.append(shortenRecipePath(hookPath).rstrip())
                with open(hookPath, "r", encoding="utf-8") as file:
                    lastLineWithExitCode = ""
                    for line in file:
                        if "EXIT CODE" in line:
                            lastLineWithExitCode = line
                    rowData.append(getResultStr(lastLineWithExitCode))
                    rowData.append(get_exit_code_chunk(lastLineWithExitCode))
                tableData.append(rowData)

    print_table_format(tableData)

    logger.info("[END recipes checks checks]")

def describe(name):
    hostname = socket.gethostname()
    colored_hostname=telemetry_logger.header_message(hostname)
    logger.info("[START recipe checks for recipe with name {0} on node {1}]\n".format(name, colored_hostname))

    recipePath = "/var/log/recipes/{0}".format(name)

    tableData = []

    rowData = []
    rowData.append(shortenRecipePath(recipePath))
    with open(recipePath, "r", encoding="utf-8") as file:
        lastLineWithExitCode = ""
        logger.info("[Print Recipe execution logs for recipe ]".format(name))
        for line in file:
            logger.info(line)
            if "EXIT CODE" in line:
                lastLineWithExitCode = line
        rowData.append(getResultStr(lastLineWithExitCode))

        rowData.append(get_exit_code_chunk(lastLineWithExitCode))
        logger.info("[END OF Print Recipe execution logs for recipe ]".format(name))

        tableData.append(rowData)
        print_table_format(tableData)

    logger.info("[END recipes checks]")

def getResultStr(line):
    if "EXIT CODE: 0" in line:
        return telemetry_logger.success_message("Success")
    else:
        return telemetry_logger.fail_message("Failure")

def get_exit_code_chunk(line):
    m = re.search("(EXIT CODE: \d*)", line)
    if m is not None:
        return m.group(1)
    return "NOT FOUND"

def shortenRecipePath(path):
    # /var/log/recipes is 17 long :)
    return path[17:]

def print_table_format(results):
    logger.info(telemetry_logger.format_table(results, ['RecipePath', 'Result', 'Exit Code']))