import sys
import shutil
import subprocess
import logging
import glob
import os
from utils import telemetry_logger, command_helper

SALT_COMMAND_PREFIX="/opt/salt_3000.8/bin/salt *"
SALT_COMMAND_PREFIX_WITH_NO_SPLIT="/opt/salt_3000.8/bin/salt '*'"
LOCAL_SALT_COMMAND_PREFIX="/opt/salt_3000.8/bin/salt-call"
SALT_KEY_COMMAND_PREFIX="/opt/salt_3000.8/bin/salt-key"

logger = logging.getLogger(__name__)

def run_salt_command(cmd, local=False, doNotSplit=False):
    full_command=None
    if local:
        full_command="%s %s" % (get_salt_local_command_prefix(), cmd)
    else:
        if doNotSplit:
            full_command="%s %s" % (get_salt_command_prefix_no_split(), cmd)
        else:
            full_command="%s %s" % (get_salt_command_prefix(), cmd)
    return command_helper.run_command(full_command, doNotSplit)

def run_salt_key_command(cmd):
    full_command="%s %s" % (get_salt_key_command_prefix(), cmd)
    return command_helper.run_command(full_command, True)

def ping(local=False, output_format=None):
    ping_cmd="test.ping" if output_format == "json" else "test.ping" # todo: json output
    test_command=None
    if local:
        test_command="%s %s" % (get_salt_local_command_prefix(), ping_cmd)
    else:
        test_command="%s %s" % (get_salt_command_prefix(), ping_cmd)
    out, err, exit_code = command_helper.run_command(test_command)
    if exit_code != 0:
        logger.debug("Exit code for salt ping: %s, Err: %s", exit_code, err)
    stroutput=str(out.rstrip().decode())
    host_state_pairs_map={}
    act=None
    for line in stroutput.splitlines():
        if line.endswith(":"):
            act=line[:-1]
            host_state_pairs_map[act]=None
        else:
            if line.strip() == "True":
                host_state_pairs_map[act]=telemetry_logger.success_message("[OK]")
            else:
                host_state_pairs_map[act]=telemetry_logger.fail_message("[NOK]")
    host_state_pairs=list(host_state_pairs_map.items())
    logger.info(telemetry_logger.format_table(host_state_pairs, ['Host', 'Status']))

def get_salt_command_prefix():
    salt_folder=get_salt_folder()
    if salt_folder:
        return os.path.join(salt_folder, "bin", "salt") + " *"
    else:
        return SALT_COMMAND_PREFIX

def get_salt_command_prefix_no_split():
    salt_folder=get_salt_folder()
    if salt_folder:
        return os.path.join(salt_folder, "bin", "salt") + " '*'"
    else:
        return SALT_COMMAND_PREFIX_WITH_NO_SPLIT

def get_salt_local_command_prefix():
    salt_folder=get_salt_folder()
    if salt_folder:
        return os.path.join(salt_folder, "bin", "salt-call")
    else:
        return LOCAL_SALT_COMMAND_PREFIX

def get_salt_key_command_prefix():
    salt_folder=get_salt_folder()
    if salt_folder:
        return os.path.join(salt_folder, "bin", "salt-key")
    else:
        return SALT_KEY_COMMAND_PREFIX

def get_salt_folder():
    result=None
    salt_folders = glob.glob("/opt/salt_*")
    for salt_folder in salt_folders:
        return salt_folder
    return result