import os
import glob
import logging

from utils import telemetry_logger
from utils import response

logger=logging.getLogger(__name__)

def list_commands(name):
    agent_commands=[]
    for command_folder in __query(name, None):
        scm_agent_command=ScmAgentCommand(command_folder)
        agent_commands.append(scm_agent_command.create_formatted_tuple())
    if agent_commands:
        responseHolder=response.ResponseHolder(tuple_content=agent_commands)
        logger.info(responseHolder.to_table(['Name', 'ID', 'Exit Code'])) 
    else:
        logger.info("No entry found!")

def describe_command(command_id, command_name, include_logs):
    for command_folder in __query(command_id, command_name):
        scm_agent_command=ScmAgentCommand(command_folder)
        scm_agent_command.describe_command(include_logs)

def __query(commandId, commandName):
    command_folder_prefix="/var/run/cloudera-scm-agent/process/"
    if commandName and commandId:
        return glob.glob('%s%s-%s' % (command_folder_prefix, commandId, commandName))
    elif commandName:
        return glob.glob('%s[0-9]*%s' % (command_folder_prefix, commandName))
    elif commandId:
        return glob.glob('%s%s*' % (command_folder_prefix, commandId))
    else:
        return glob.glob('%s[0-9]*' % command_folder_prefix)

class ScmAgentCommand:

    def __init__(self, command_folder):
        self.command_folder=command_folder
        self.__fill_fields(command_folder)
    
    def __fill_fields(self, command_folder):
        command_folder_prefix="/var/run/cloudera-scm-agent/process/"
        command_folder_suffix=self.__remove_prefix(command_folder, command_folder_prefix)
        splitted_by_command_id=command_folder_suffix.split("-", 1)
        exit_code_file=os.path.join(command_folder, "exit_code")
        self.command_id=splitted_by_command_id[0]
        self.command_name=splitted_by_command_id[1].split(os.path.sep, 1)[0]
        self.exit_code="0"
        with open(exit_code_file, 'r') as f:
            exit_code_str = f.read().replace('\n', '')
            if exit_code_str:
                self.exit_code = exit_code_str

    def describe_command(self, include_logs):
        logger.info("==============")
        logger.info("Command Id: %s" % self.command_id)
        logger.info("Command Name: %s" % self.command_name)
        logger.info("Exit Code: %s" % self.exit_code)
        if include_logs:
            logger.info("StdOut:")
            logger.info("--------------")
            with open(os.path.join(self.command_folder, "logs", "stdout.log"), 'r') as f:
                logger.info(f.read())
            logger.info("--------------")
            logger.info("StdErr:")
            logger.info("--------------")
            with open(os.path.join(self.command_folder, "logs", "stderr.log"), 'r') as f:
                logger.info(f.read())
            logger.info("--------------")
        logger.info("==============")

    def create_formatted_tuple(self):
        exit_code_formatted=None
        if self.exit_code == "0":
            exit_code_formatted=telemetry_logger.success_message(self.exit_code)
        else:
            exit_code_formatted=telemetry_logger.fail_message(self.exit_code)
        return (self.command_name, self.command_id, exit_code_formatted)

    def __remove_prefix(self, s, prefix):
        return s[len(prefix):] if s.startswith(prefix) else s