#!/usr/bin/env python
import logging
import psutil
import sys
from google.protobuf.json_format import MessageToDict
from nodestatus.generated import nodestatus_pb2 as NodeStatus
from utils import response, command_helper
from metricscollector.host_metrics_collector import HostMetricsCollector

logger = logging.getLogger(__name__)

def collect_connections_details(pid_filter, raddr_ip_filter, raddr_port_filter, laddr_ip_filter, laddr_port_filter, status_filter, format, verbose, pretty=False):
    try:
        status_maps={}
        connections=list()
        jsonResponse={}
        for conn in psutil.net_connections():
            conn_map={}
            fd=str(conn.fd) if conn.fd != -1 else "NONE"
            pid=str(conn.pid) if conn.pid else "NONE"
            raddr=conn.raddr
            raddr_ip="NONE"
            raddr_port="NONE"
            if raddr:
                raddr_ip=str(raddr.ip)
                raddr_port=str(raddr.port)
            laddr=conn.laddr
            laddr_ip="NONE"
            laddr_port="NONE"
            if laddr:
                laddr_ip=str(laddr.ip)
                laddr_port=str(laddr.port)
            status=conn.status
            conn_map["pid"]=pid
            conn_map["raddr_ip"]=raddr_ip
            conn_map["raddr_port"]=raddr_port
            conn_map["laddr_ip"]=laddr_ip
            conn_map["laddr_port"]=laddr_port
            conn_map["fd"]=fd
            conn_map["status"]=status
            if verbose and __notFilteredOut(conn_map, pid_filter, raddr_ip_filter, raddr_port_filter, laddr_ip_filter, laddr_port_filter, status_filter):
                if format == "json":
                    connections.append(conn_map)
                else:
                    connections.append((pid, laddr_ip, laddr_port, raddr_ip, raddr_port, fd, status))
            if str(conn.status) != "NONE" and __notFilteredOut(conn_map, pid_filter, raddr_ip_filter, raddr_port_filter, laddr_ip_filter, laddr_port_filter, status_filter):
                if conn.status not in status_maps:
                    status_maps[conn.status]=1
                else:
                    status_maps[conn.status]=status_maps[conn.status] + 1
        if format == "json":
            jsonResponse["statuses"]=status_maps
        else:
            logger.info("Connections - States:")
            logger.info(response.ResponseHolder(map_content=status_maps).to_table())
        if verbose:
            if format == "json":
                jsonResponse["connections"]=connections
            else:
                headers=["PID", "Local IP", "Local Port", "Remote IP", "Remote Port", "File Descriptor", "Status"]
                logger.info("Connections - Details:")
                logger.info(response.ResponseHolder(tuple_content=connections).to_table(headers=headers))
        if format == "json":
            logger.info(response.ResponseHolder(map_content=jsonResponse).to_json(pretty, True))
    except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
       logger.error("Cannot read nework connections (requires root access)")
       sys.exit(1)

def __notFilteredOut(conn_map, pid_filter, raddr_ip_filter, raddr_port_filter, laddr_ip_filter, laddr_port_filter, status_filter):
    result = True
    if pid_filter and conn_map["pid"] != pid_filter:
        result = False
    elif raddr_ip_filter and conn_map["raddr_ip"] != raddr_ip_filter:
        result = False
    elif raddr_port_filter and conn_map["raddr_port"] != raddr_port_filter:
        result = False
    elif laddr_ip_filter and conn_map["laddr_ip"] != laddr_ip_filter:
        result = False
    elif laddr_port_filter and conn_map["laddr_port"] != laddr_port_filter:
        result = False
    elif status_filter and conn_map["status"] != status_filter:
        result = False   
    return result

def collect_process_details(top, format, pretty=False):
    procs=[]
    temp_procs=[]
    hostMetricsCollector=HostMetricsCollector()
    top_processes_map={}
    if sys.platform == "linux":
        top_cmd='ps -eo pid,%%mem,%%cpu --sort=-%%mem,%%cpu | head -%s | sed "1 d" | sed "s/^ *//" | sed "s/[[:space:]]\{1,\}/,/g"' % (str(top+1))
        out,err,code=command_helper.run_command(top_cmd, True)
        if (str(code).rstrip()) == "0":
             stroutput=str(out.rstrip().decode())
             for line in stroutput.splitlines():
                 splitted=line.split(",")
                 if len(splitted) == 3:
                    top_processes_map[str(splitted[0])]=(str(splitted[1]), str(splitted[2]))
    for proc in psutil.process_iter():
        try:
            proc_map={}
            proc_map['pid']=proc.pid
            proc_map['name']=proc.name()
            proc_map['cmdline_start']=" ".join(proc.cmdline()[:2])
            if sys.platform == "linux" and str(proc.pid) in top_processes_map:
                proc_mem_cpu_pair=top_processes_map[str(proc.pid)]
                proc_map['memory_percent']=proc_mem_cpu_pair[0]
                proc_map['cpu_percent']=proc_mem_cpu_pair[1]
            else:
                proc_map['memory_percent']=proc.memory_percent()
            proc_map['user']=proc.username()
            proc_map['vms']=proc.memory_info().vms
            proc_map['rss']=proc.memory_info().rss
            proc_map['open_files']=len(proc.open_files())
            __format_if_available(hostMetricsCollector, proc_map, ["vms", "rss"])
            if sys.platform == "linux":
                if str(proc.pid) in top_processes_map:
                    temp_procs.append(proc_map)
            else:
                temp_procs.append(proc_map)
        except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
           pass
    for temp_proc in temp_procs:
        number_of_connections=-1
        try:
            p=psutil.Process(temp_proc["pid"])
            number_of_connections=len(p.connections())
        except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
           pass
        temp_proc["open_conns"]=number_of_connections
        if format == "json":
            procs.append(temp_proc)
        else:
            if sys.platform == "linux":
                procs.append((temp_proc['pid'], temp_proc['name'], temp_proc['user'], temp_proc['memory_percent'], temp_proc['cpu_percent'], temp_proc['vms'], temp_proc['rss'], temp_proc['open_files'], temp_proc["open_conns"]))
            else:
                procs.append((temp_proc['pid'], temp_proc['name'], temp_proc['user'], temp_proc['memory_percent'], temp_proc['vms'], temp_proc['rss'], temp_proc['open_files'], temp_proc["open_conns"]))
    headers=["PID", "Name", "User", "MEMORY%", "VMS", "RSS", "Open Files", "Open Connections"]
    if sys.platform == "linux":
        headers=["PID", "Name", "User", "MEMORY%", "CPU%", "VMS", "RSS", "Open Files", "Open Connections"]
    top_memory_list = sorted(procs, key=lambda k: k['memory_percent'], reverse=True)[:top] if format == "json" else sorted(procs, key=lambda k: k[4], reverse=True)[:top]
    if format == "json":
        logger.info(response.ResponseHolder(tuple_content=top_memory_list).to_json(pretty, True))
    else:
        logger.info("Top %s Memory/CPU consumer processes:" % top)
        logger.info(response.ResponseHolder(tuple_content=top_memory_list).to_table(headers=headers))

def collect_system_metrics(format, verbose, pretty=False):
    hostMetricsCollector=HostMetricsCollector(verbose=verbose)
    systemMetrics=hostMetricsCollector.collect_all()
    if format == "json":
        hostMetrics=MessageToDict(systemMetrics)
        logger.info(response.ResponseHolder(map_content=hostMetrics).to_json(pretty, True))
    else:
        hostMetrics=MessageToDict(systemMetrics, including_default_value_fields=True)
        diskMetrics=hostMetrics["disk"]
        if "partitions" in diskMetrics:
            flatPartitionData=list()
            for partitionData in diskMetrics["partitions"]:
                partitionData["total"]=hostMetricsCollector.sizeof_fmt(int(partitionData['usage']["total"]))
                partitionData["used"]=hostMetricsCollector.sizeof_fmt(int(partitionData['usage']["used"]))
                partitionData["free"]=hostMetricsCollector.sizeof_fmt(int(partitionData['usage']["free"]))
                partitionData["percent"]=str(partitionData['usage']["percent"]) + "%"
                del partitionData['usage']
                del partitionData['opts']
                flatPartitionData.append((partitionData["device"], partitionData["mountpoint"], partitionData["fstype"], partitionData["maxfile"], partitionData["maxpath"], partitionData["total"], partitionData["used"], partitionData["free"], partitionData["percent"]))
            logger.info("Disk - Partitions:")
            headers=["Device", "Mountpoint", "Fstype", "Maxfile", "Maxpath", "Total", "Used", "Free", "Percent"]
            logger.info(response.ResponseHolder(tuple_content=flatPartitionData).to_table(headers=headers))
        if "io" in diskMetrics:
            logger.info("Disk - I/O:")
            logger.info(response.ResponseHolder(map_content=diskMetrics["io"]).to_table())
        if "topVarlog" in diskMetrics:
            topVarLog=diskMetrics["topVarlog"]
            topVarLogTupleList=list()
            for item in topVarLog:
                topVarLogTupleList.append((item["path"], item["size"]))
            logger.info("Disk - Top largest folders in /var/log:")
            logger.info(response.ResponseHolder(tuple_content=topVarLogTupleList).to_table(headers=["Path", "Size"]))
        networkMetrics=hostMetrics["network"]
        if "io" in networkMetrics:
            logger.info("Network - I/O:")
            logger.info(response.ResponseHolder(map_content=networkMetrics["io"]).to_table())
        if "connections" in networkMetrics:
            logger.info("Network - Connections:")
            logger.info(response.ResponseHolder(map_content=networkMetrics["connections"]).to_table())
        memoryMetrics=hostMetrics["memory"]
        if "virtual_memory" in memoryMetrics:
            logger.info("Memory - Virtual memory:")
            formattedVirtualMemory=__format_if_available(hostMetricsCollector, memoryMetrics["virtual_memory"], ["total", "available","used", "free", "active", "inactive", "shared", "wired"])
            logger.info(response.ResponseHolder(map_content=formattedVirtualMemory).to_table())
        if "swap_memory" in memoryMetrics:
            logger.info("Memory - Swap memory:")
            formattedSwapMemory=__format_if_available(hostMetricsCollector, memoryMetrics["swap_memory"], ["total", "used", "free"])
            logger.info(response.ResponseHolder(map_content=formattedSwapMemory).to_table())
        cpuMetrics=hostMetrics["cpu"]
        if "count" in cpuMetrics and "logical_count" in cpuMetrics:
            logger.info("CPU - Count:")
            cpuCountResp=list()
            cpuCountResp.append((str(cpuMetrics["count"]), str(cpuMetrics["logical_count"])))
            logger.info(response.ResponseHolder(tuple_content=cpuCountResp).to_table(headers=["Count", "Logical Count"]))
        if "times" in cpuMetrics:
            logger.info("CPU - Times:")
            timesWithPercentMap={}
            for k in cpuMetrics["times"]:
                timesWithPercentMap[k]=str(round(cpuMetrics["times"][k], 2)) + " %"
            logger.info(response.ResponseHolder(map_content=timesWithPercentMap).to_table())
        if "stats" in cpuMetrics:
            logger.info("CPU - Stats:")
            logger.info(response.ResponseHolder(map_content=cpuMetrics["stats"]).to_table())
        if "freq" in cpuMetrics:
            logger.info("CPU - Frequency:")
            logger.info(response.ResponseHolder(map_content=cpuMetrics["freq"]).to_table())
        if "load_avg" in cpuMetrics:
            logger.info("CPU - Load AVG:")
            loadAvgMap={}
            loadAvgMap["Last 1 min"]=round(cpuMetrics["load_avg"][0], 2)
            loadAvgMap["Last 5 min"]=round(cpuMetrics["load_avg"][1], 2)
            loadAvgMap["Last 15 min"]=round(cpuMetrics["load_avg"][2], 2)
            logger.info(response.ResponseHolder(map_content=loadAvgMap).to_table())

def __format_if_available(hostMetricsCollector, field_value_map, field_list=[]):
    for key in field_value_map:
        if key in field_list and key in field_value_map and field_value_map[key]:
            field_value_map[key]=hostMetricsCollector.sizeof_fmt(field_value_map[key])
    return field_value_map