import logging
import fileinput
import re
import sys
import os
import tempfile
from filecollector.external_command import ExternalCommand

logger=logging.getLogger(__name__)

class FileAnonymizer:

    def __init__(self, path, rules, label, usePerlScript=True):
        self.path=path
        self.rules=self.__get_filtered_rules(rules, label)
        self.usePerlScript=usePerlScript

    def redact(self):
        if not self.rules:
            return
        if self.usePerlScript:
            logger.debug("Run anonymization on file (perl script): %s" % self.path)
            perlScriptPath=None
            if getattr(sys, 'frozen', False):
                perlScriptPath=os.path.join(sys._MEIPASS, "files/replace_strings.pl")
            else:
                parent=os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
                perlScriptPath=os.path.join(parent, "scripts", "replace_strings.pl")
            if perlScriptPath:
                logger.debug("Perl script location for anonymization: %s" % perlScriptPath)
                self.__run_pl_script(perlScriptPath)
        else:
            logger.debug("Run anonymization on file (built-in): %s" % self.path)
            for line in fileinput.FileInput(self.path, inplace=1):
                for rule in self.rules:
                    line = re.sub(rule["pattern"], rule["replacement"], line.rstrip())
                print(line)

    def __run_pl_script(self, perlScriptPath):
        pcre_replace_patterns=[]
        for rule in self.rules:
            pcre_replace_patterns.append("s/%s/%s/gi" % (rule["pattern"], rule["replacement"]))
        generated_replacements_content="\n".join(pcre_replace_patterns)
        with tempfile.NamedTemporaryFile() as fp:
            fp.seek(0)
            fp.write(generated_replacements_content.encode("utf-8"))
            fp.flush()
            command="perl %s -nb -nq -S %s -f %s" % (perlScriptPath, fp.name, self.path)
            result=ExternalCommand(command).run()
            if len(result) == 3 and result[2] == 0:
                logger.debug("Running external command with successful response.")

    def __get_filtered_rules(self, rules, label):
        final_rules=[]
        for rule in rules:
            if "labels" in rule and label not in rule["labels"]:
                continue
            final_rules.append(rule)
        return final_rules