import logging
import shutil
import os
from filecollector.external_command import ExternalCommand

logger=logging.getLogger(__name__)

class Archiver():

    def __init__(self, source, destination, compressFormat, deleteSource=False, useExternalCommand=False):
        self.source=source
        self.destination=destination
        self.compressFormat=compressFormat
        self.extension="zip"
        if compressFormat == "tar":
            self.extension = "tar"
        elif compressFormat == "gztar":
            self.extension = "tar.gz"
        elif compressFormat == "bztar":
            self.extension = "tar.bz2"
        self.deleteSource=deleteSource
        self.useExternalCommand=useExternalCommand
    
    def get_extension(self):
        return self.extension

    def make_archive(self):
        if not os.path.exists(self.source):
            logger.debug("Source does not exist, skip compression: %s", self.source)
            return
        name = os.path.basename(self.destination)
        archive_from = os.path.dirname(self.source)
        archive_to = os.path.basename(self.source.strip(os.sep))
        if self.useExternalCommand:
            self.__compress_with_external_command(name, archive_from, archive_to)
        else: 
            shutil.make_archive(name, self.compressFormat, archive_from, archive_to)
        shutil.move("%s.%s" % (name, self.extension), "%s.%s" % (self.destination, self.extension))
        if self.deleteSource:
            shutil.rmtree(self.source)
    
    def __compress_with_external_command(self, name, archive_from, archive_to):
        if self.extension == "zip":
            ExternalCommand("tar -cZf %s.zip -C %s %s" % (archive_to, archive_from, name)).run()
        elif self.extension == "tar":
            ExternalCommand("tar -cf %s.tar -C %s %s" % (archive_to, archive_from, name)).run()
        elif self.extension == "tar.gz":
            ExternalCommand("tar -czf %s.tar.gz -C %s %s" % (archive_to, archive_from, name)).run()
        elif self.extension == "tar.bz2":
            ExternalCommand("tar -cjf %s.tar.gz -C %s %s" % (archive_to, archive_from, name)).run()