import os
import glob
import logging

from filecollector.config import Config

logger=logging.getLogger(__name__)

class CollectedFilesHolder:

    def __init__(self, fileObj, startTime, endTime, sortFilesByDate):
        fileObjConfig=Config(fileObj)
        self.path=fileObjConfig.get_str_key("path")
        self.label=fileObjConfig.get_str_key("label")
        self.excludes=fileObjConfig.get_key("excludes", [])
        self.startTime=startTime
        self.endTime=endTime
        self.sortFilesByDate=sortFilesByDate
        self.commonConfig=fileObjConfig
        self.totalSize=0
        self.files=self.__gather_files()

    def get_files(self):
        return self.files
    
    def get_total_size(self):
        return self.totalSize

    def get_common_config(self):
        return self.commonConfig
    
    def __gather_files(self):
        allfiles=sorted(glob.glob(self.path, recursive=True), key=os.path.getmtime) if self.sortFilesByDate else glob.glob(self.path)
        exclude_files=self.__get_excludes(self.excludes)
        files_to_collect=[]
        for file in allfiles:
            if file in exclude_files:
                continue
            file_object=CollectedFile(file, self.label)
            if self.__is_file_not_in_date_rage(file_object, self.startTime, self.endTime):
                continue
            self.totalSize=self.totalSize + file_object.get_size()
            files_to_collect.append(file_object)
        return files_to_collect
    
    def __get_excludes(self, paths):
        exclude_files=[]
        if paths:
            for filepath in paths:
                files=glob.glob(filepath)
                for file in files:
                    exclude_files.append(file)
                    logger.debug("file %s will be excluded from processing." % file)
        return exclude_files
    
    def __is_file_not_in_date_rage(self, file_obj, start_time, end_time):
        result=False
        if start_time or end_time:
            file_path=file_obj.get_abs_path()
            creationDate=file_obj.get_creation_date()
            lastModifiedDate=file_obj.get_last_modified_date()
            logger.debug("file '%s' creation date: %s." % (file_path, creationDate))
            logger.debug("file '%s' last modification date: %s." % (file_path, lastModifiedDate))
            if start_time and lastModifiedDate < (start_time / 1000.0):
                logger.debug("skipping processing file '%s' as last modification time is smaller than 'start-time' parameter" % file_path)
                result=True
            elif end_time and (end_time / 1000.0) < creationDate:
                logger.debug("skipping processing file '%s' as creation time is larger than 'end-time' parameter" % file_path)
                result=True
            else:
                logger.debug("file '%s' is in the right date range (based on creation/last modification datestamps)" % file_path)
        return result

class CollectedFile:

    def __init__(self, path, label):
        self.path=path
        self.label=label
        self.abs_path=os.path.abspath(path)
        self.creation_date=self.__creation_date(self.abs_path)
        self.last_modified=os.stat(self.abs_path).st_mtime
        self.size=os.stat(self.abs_path).st_size

    def get_path(self):
        return self.path
    
    def get_abs_path(self):
        return self.abs_path

    def get_last_modified_date(self):
        return self.last_modified

    def get_creation_date(self):
        return self.creation_date

    def get_size(self):
        return self.size

    def __creation_date(self, file_path):
        stat = os.stat(file_path)
        try:
            return stat.st_birthtime
        except AttributeError:
            return stat.st_mtime