import os
import logging
from fluent import sender
from fluent import event

logger=logging.getLogger(__name__)

class EventProcessor:

    def __init__(self, host, port, base_tag, additional_fields = {}, additional_fields_key = "additionalContext", message_field="message", include_time=False):
        self.host = host
        self.port = port
        self.base_tag = base_tag
        self.message_field = message_field
        self.include_time = include_time
        self.additional_fields = {}
        self.additional_fields_key = additional_fields_key
        if bool(additional_fields):
            self.additional_fields = {}
            for key, val in additional_fields.items():
                self.additional_fields[key]=val
        if host and port:
            self.fluentSender = sender.FluentSender(base_tag, host=host, port=port)
        else:
            self.fluentSender = sender.FluentSender(base_tag)

    def process(self, name, path, real_path):
        with open(real_path, 'r', buffering=100000) as infile:
            if "*" in name:
                replaced_path=path.replace(os.sep, ".")
                name=name.replace("*", replaced_path)
            for line in infile:
                fields = {self.message_field: line}
                if bool(self.additional_fields):
                    fields[self.additional_fields_key]=self.additional_fields
                if self.include_time:
                    self.fluentSender.emit_with_time(name, time.time(), fields)
                else:
                    self.fluentSender.emit(name, fields)

    def close(self):
        self.fluentSender.close()