import argparse
import logging
import time
import uuid
import sys
import yaml

def parse_args():
    parser = argparse.ArgumentParser(
        description='Python script to generate periodic metrics from a source '
                    'metrics collector')
    parser.add_argument('--config', type=str, required=True,
                        help='Path to metrics collector configuration')
    args = parser.parse_args()
    return args

def main():
    args = parse_args()
    run(args.config)

def run(config_path):
    with open(config_path) as file:
        config = yaml.load(file, yaml.SafeLoader)
        if "clouderaManager" in config:
            from .cm_metrics_collector import ClouderaManagerMetricsCollector
            ClouderaManagerMetricsCollector(config).start()
        elif "freeIpa" in config:
            # TODO
            pass
        elif "systemMetricsCollector" in config:
            from .system_metrics_collector import SystemMetricsCollectorAgent
            SystemMetricsCollectorAgent(config).start()
        else:
            print("Configuration should contain System Metrics / Cloudera Manager or FreeIPA section. Exiting ...")
            sys.exit(1)

if __name__ == '__main__':
   main()