import json
import logging
from logging.handlers import RotatingFileHandler

class MetricsLogger:
    def __init__(self, name, path, max_bytes, backup_count, debug=False):
        fmt = '%(message)s'
        logging.basicConfig(
                 level=logging.DEBUG,
                 format=fmt,
        )
        formatter = logging.Formatter(fmt)
        handler = RotatingFileHandler(path, maxBytes=max_bytes, backupCount=backup_count, mode='a')
        handler.setLevel(logging.DEBUG)
        handler.setFormatter(formatter)
        if logging.getLogger(name).hasHandlers():
            logging.getLogger(name).handlers.clear()
        self.metrics_logger = logging.getLogger(name)
        self.metrics_logger.addHandler(handler)

    def process(self, event):
        self.metrics_logger.debug(json.dumps(event))

class MetricsStatusLogger:
    def __init__(self, file):
        self.file=file

    def process(self, event):
        with open(self.file, "w") as metrics_file:
            json.dump(event, metrics_file, indent=2, sort_keys=True)

class NullProcessor:
    def __init__(self):
        pass

    def process(self, event):
        pass