import glob
import os
import logging
import sys
import gzip

logger=logging.getLogger(__name__)

def get_content(filename):
    if filename.endswith("gz"):
        with gzip.open(filename, 'rb') as f:
            return f.read()
    else:
        with open(filename) as f:
            return f.read()

def upload(account, container, file, paths, location, use_msi, verbose, resource='https://storage.azure.com/', url_suffix='.dfs.core.windows.net'):
    from azure.identity import ManagedIdentityCredential, DefaultAzureCredential
    from azure.storage.filedatalake import DataLakeServiceClient
    credential=None
    if use_msi:
        credential=ManagedIdentityCredential()
    else:
        credential=DefaultAzureCredential()
    
    service_client = DataLakeServiceClient(account_url="{}://{}{}".format("https", account, url_suffix), credential=credential, logging_enable=verbose)
    if paths:
        logger.debug("Use matcher to search for files: %s" % (paths))
        files = glob.glob(paths)
        for file_to_upload in files:
            logger.debug("File to upload: %s" % (file_to_upload))
            base_file=os.path.basename(file_to_upload)
            upload_file_path = location + base_file if location.endswith("/") else location + "/" + base_file
            upload_file_path=upload_file_path if upload_file_path.startswith("/") else "/" + upload_file_path
            logger.debug("Upload destination: %s" % (upload_file_path))
            file_client=service_client.get_file_client(container, upload_file_path)
            file_client.create_file()
            file_client.upload_data(get_content(file_to_upload), overwrite=True)
    else:
        logger.debug("File to upload: %s" % (file))
        upload_file_path=location + os.path.basename(file) if location.endswith("/") else location
        upload_file_path=upload_file_path if upload_file_path.startswith("/") else "/" + upload_file_path
        logger.debug("Upload destination: %s" % (upload_file_path))
        file_client=service_client.get_file_client(container, upload_file_path)
        file_client.create_file()
        file_client.upload_data(get_content(file), overwrite=True)
