import os
import glob
import errno
import logging
from google.cloud import storage

logger=logging.getLogger(__name__)

def upload_file(bucket, source_file, dest_file):
    blob = bucket.blob(dest_file)
    blob.upload_from_filename(source_file)

def upload(bucket_name, file, paths, location, credential):
    storage_client = storage.Client.from_service_account_json(credential) if credential else storage.Client()
    bucket = storage_client.bucket(bucket_name)
    if paths:
        logger.debug("Use matcher to search for files: %s" % (paths))
        files = glob.glob(paths)
        for file_to_upload in files:
            logger.debug("File to upload: %s" % (file_to_upload))
            base_file=os.path.basename(file_to_upload)
            upload_file_path = location + base_file if location.endswith("/") else location + "/" + base_file
            logger.debug("Upload destination: %s" % (upload_file_path))
            upload_file(bucket, file_to_upload, upload_file_path)
    else:
        logger.debug("File to upload: %s" % (file))
        upload_file_path=location + os.path.basename(file) if location.endswith("/") else location
        logger.debug("Upload destination: %s" % (upload_file_path))
        upload_file(bucket, file, upload_file_path)