from requests.packages.urllib3 import Retry
import os
import requests
import logging

logger=logging.getLogger(__name__)

TOKEN_TTL_SECONDS = 60
TOKEN_HEADER = "X-aws-ec2-metadata-token"
TOKEN_HEADER_TTL = "X-aws-ec2-metadata-token-ttl-seconds"

def get_aws_instance_region(use_token=False):
    return __get_aws_ec2_metadata("meta-data/placement/region", use_token)

def get_aws_instance_domain(use_token=False):
    return __get_aws_ec2_metadata("meta-data/services/domain", use_token)

def __get_aws_ec2_metadata(uri_suffix, use_token=False):
    if "HTTPS_PROXY" in os.environ:
        del os.environ["HTTPS_PROXY"]
    if "HTTP_PROXY" in os.environ:
        del os.environ["HTTP_PROXY"]
    if "https_proxy" in os.environ:
        del os.environ["https_proxy"]
    if "http_proxy" in os.environ:
        del os.environ["http_proxy"]
    url_prefix='http://169.254.169.254/latest'
    full_url="%s/%s" % (url_prefix, uri_suffix)
    if use_token:
        logger.debug("Use ec2 metadata service with gathering metadata service token first.")
        token_url="%s/%s" % (url_prefix, "api/token")
        try:
            token_response=requests.put(token_url, timeout=(2, 5), headers={TOKEN_HEADER_TTL: str(TOKEN_TTL_SECONDS)})
            if token_response.status_code != 200:
                return None
            else:
                token=token_response.text
                r = requests.get(full_url, timeout=(2, 5), headers={TOKEN_HEADER: str(token), 'Connection':'close'})
        except (requests.exceptions.ConnectTimeout, requests.exceptions.ConnectionError) as err:
            logger.debug("Connection to AWS EC2 Metadata timed out: " + str(err.__class__.__name__))
            logger.debug("Is this an EC2 instance? Is the AWS metadata endpoint blocked or disabled? (http://169.254.169.254/)")
            return None
    else:
        logger.debug("Use ec2 metadata service without gathering metadata service token.")
        try:
            r = requests.get(full_url, timeout=(2, 5), headers={'Connection':'close'})
        except (requests.exceptions.ConnectTimeout, requests.exceptions.ConnectionError) as err:
            logger.debug("Connection to AWS EC2 Metadata timed out: " + str(err.__class__.__name__))
            logger.debug("Is this an EC2 instance? Is the AWS metadata endpoint blocked or disabled? (http://169.254.169.254/)")
            return None
    if r.status_code == 404 or r.status_code == 500:
        return None
    if r.status_code == 400 or r.status_code == 401 or r.status_code == 403:
        if not use_token:
            return __get_aws_ec2_metadata(uri_suffix, True)
        else:
            return None
    return r.text