import logging
import os
from .proxy_settings import ProxySettings

logger=logging.getLogger(__name__)

CDP_LOGGING_AGENT_CONF_PREFIX="/etc/cdp-logging-agent"
TD_AGENT_CONF_PREFIX="/etc/td-agent"

# CDP_LOGGING_AGENT_CONF_PREFIX="examples/metering"

class LoggingAgentConfigHolder(object):

    def __init__(self):
        self.output_configs=self.__resolve_output_configs()
        self.output_databus_configs=self.__resolve_output_databus_configs()
        self.output_metering_configs=self.__resolve_output_metering_configs()

    def get_output_configs(self):
        return self.output_configs

    def get_metering_output_configs(self):
        return self.output_metering_configs

    def get_databus_endpoint(self):
        databus_conf=self.get_databus_output_configs()
        metering_conf=self.get_metering_output_configs()
        if "endpoint" in metering_conf:
            return  metering_conf["endpoint"]
        elif "endpoint" in databus_conf:
            return  databus_conf["endpoint"]
        else:
            return False

    def get_databus_output_configs(self):
        return self.output_databus_configs

    def is_any_databus_config(self):
        return True if self.output_databus_configs or self.output_metering_configs else False

    def get_proxy_configs(self):
        proxySettings=None
        if self.is_any_databus_config():
            proxyUrl=None
            proxyUser=''
            proxyPassword=''
            if self.output_metering_configs:
                proxyUrl=self.output_metering_configs["proxy_url"] if "proxy_url" in self.output_metering_configs else None
                proxyUser=self.output_metering_configs["proxy_username"] if "proxy_username" in self.output_metering_configs else ''
                proxyPassword=self.output_metering_configs["proxy_password"] if "proxy_password" in self.output_metering_configs else ''
            elif self.output_databus_configs:
                proxyUrl=self.output_databus_configs["proxy_url"] if "proxy_url" in self.output_databus_configs else None
                proxyUser=self.output_databus_configs["proxy_username"] if "proxy_username" in self.output_databus_configs else ''
                proxyPassword=self.output_databus_configs["proxy_password"] if "proxy_password" in self.output_databus_configs else ''
            if proxyUrl:
                proxySettings=ProxySettings(proxy_url=proxyUrl, proxy_user=proxyUser, proxy_password=proxyPassword)
        return proxySettings

    def __resolve_output_configs(self):
        return self.__resolve_logging_configs("output.conf")

    def __resolve_output_databus_configs(self):
        return self.__resolve_logging_configs("output_databus.conf")

    def __resolve_output_metering_configs(self):
        return self.__resolve_logging_configs("databus_metering.conf")

    def __resolve_logging_configs(self, config_name):
        result={}
        logging_agent_conf=None
        if os.path.exists(CDP_LOGGING_AGENT_CONF_PREFIX):
            logging_agent_conf = "%s/%s" % (CDP_LOGGING_AGENT_CONF_PREFIX, config_name)
        elif os.path.exists(TD_AGENT_CONF_PREFIX):
            logging_agent_conf = "%s/%s" % (TD_AGENT_CONF_PREFIX, config_name)
        if logging_agent_conf and os.path.exists(logging_agent_conf):
            with open(logging_agent_conf) as f:
                for line in f:
                    lineStripped=line.strip()
                    splitted=lineStripped.split()
                    if splitted and not splitted[0].startswith("<") and not splitted[0].startswith("#"):
                        if len(splitted) > 1:
                            result[splitted[0]]=splitted[1].replace('"', '')
        return result
