import os

class ProxySettings:

    def __init__(self, full_proxy_url=None, proxy_url=None, proxy_user=None, proxy_password=None):
        self.proxy_url=None
        self.proxy_user=None
        self.proxy_password=None
        if proxy_url:
            self.full_proxy_url=self.__create_full_proxy_url(proxy_url, proxy_user, proxy_password)
        elif full_proxy_url:
            self.full_proxy_url=full_proxy_url
        else:
            self.full_proxy_url = os.getenv("HTTPS_PROXY") if os.getenv("HTTPS_PROXY") is not None else None
            self.full_proxy_url = os.getenv("HTTP_PROXY") if not self.full_proxy_url and os.getenv("HTTP_PROXY") is not None else self.full_proxy_url
            self.full_proxy_url = os.getenv("https_proxy") if not self.full_proxy_url and os.getenv("https_proxy") is not None else self.full_proxy_url
            self.full_proxy_url = os.getenv("http_proxy") if not self.full_proxy_url and os.getenv("http_proxy") is not None else self.full_proxy_url
        if self.full_proxy_url:
            if "@" not in self.full_proxy_url:
                self.proxy_url=self.full_proxy_url
            else:
                splitted_url=self.full_proxy_url.split("://")
                scheme=splitted_url[0]
                if len(splitted_url) > 1:
                    url_without_scheme=splitted_url[1]
                    splitted_with_dot=url_without_scheme.split("@")
                    auth=splitted_with_dot[0].split(":")
                    self.proxy_user=auth[0]
                    self.proxy_password=auth[1] if len(auth) > 1 else ""
                    if len(splitted_with_dot) > 1:
                        self.proxy_url = "%s://%s" % (scheme, splitted_with_dot[1])
    
    def get_full_proxy_url(self):
        return self.full_proxy_url
    
    def get_url(self):
        return self.proxy_url

    def get_user(self):
        return self.proxy_user

    def get_password(self):
        return self.proxy_password

    def get_host_and_port(self):
        if self.proxy_url:
            splitted_url=self.proxy_url.split("://")
            port="80"
            host=""
            scheme="http"
            host_part=None
            if len(splitted_url) > 1:
                scheme=splitted_url[0]
                host_part=splitted_url[1]
            else:
                host_part=splitted_url[0]
            if host_part:
                host_and_port_arr=host_part.split(":")
                if len(host_and_port_arr) > 1:
                    port=host_and_port_arr[1]
                host=host_and_port_arr[0]
            return host, port
        else:
            return "", ""

    def __create_full_proxy_url(self, proxy_url, proxy_user, proxy_password):
        if proxy_url:
            splittedProxyUrl=proxy_url.split("://")
            schema=splittedProxyUrl[0]
            authString=""
            if proxy_user:
                authString="%s:@" % (proxy_user) if proxy_password is None else "%s:%s@" % (proxy_user, proxy_password)
            return "%s://%s%s" % (schema, authString, splittedProxyUrl[1])   
        else:
            return None   
