import logging
import logging.handlers
import sys
import os
from tabulate import tabulate

class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'

def setup_logger(logger_name=None, file_log_level=logging.DEBUG, file_name='cdp-telemetry.log', dir='/var/log/cdp-telemetry'):
    log_file=os.getenv('CDP_TELEMETRY_LOGGER_FILENAME', file_name)
    dir_resolved=os.getenv('CDP_TELEMETRY_LOGGER_DIR', dir)
    if os.path.exists(dir_resolved):
        log_file=os.path.join(dir_resolved, os.getenv('CDP_TELEMETRY_LOGGER_FILENAME', file_name))
    
    if logging.getLogger(logger_name).hasHandlers():
        logging.getLogger(logger_name).handlers.clear()
    
    logging.getLogger(logger_name).setLevel(logging.NOTSET)

    disableStdOut=bool(os.getenv('CDP_TELEMETRY_CONSOLE_LOGGING', False))
    if not disableStdOut:
        console = logging.StreamHandler(sys.stdout)
        console.setLevel(logging.INFO)
        formater = logging.Formatter('%(message)s')
        console.setFormatter(formater)
        logging.getLogger(logger_name).addHandler(console)

    rotatingHandler = logging.handlers.RotatingFileHandler(filename=log_file, maxBytes=10485760, backupCount=10)
    rotatingHandler.setLevel(file_log_level)
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(threadName)s - %(message)s')
    rotatingHandler.setFormatter(formatter)
    logging.getLogger(logger_name).addHandler(rotatingHandler)

def success_message(msg):
    return "%s%s%s" % (bcolors.OKGREEN, msg, bcolors.ENDC)

def fail_message(msg):
    return "%s%s%s" % (bcolors.FAIL, msg, bcolors.ENDC)

def header_message(msg):
    return "%s%s%s" % (bcolors.HEADER, msg, bcolors.ENDC)

def bold_message(msg):
    return "%s%s%s" % (bcolors.BOLD, msg, bcolors.ENDC)

def format_table(tuple_list, headers):
    return tabulate(tuple_list, headers=headers, tablefmt='pretty')

def format_table_with_format(tuple_list, headers, format):
    return tabulate(tuple_list, headers=headers, tablefmt=format)

def remove_colors(input_str):
    remove_list=[bcolors.HEADER, bcolors.OKGREEN, bcolors.WARNING, bcolors.FAIL, bcolors.ENDC, bcolors.BOLD, bcolors.UNDERLINE]
    for rem in remove_list: 
        input_str = input_str.replace(rem, '') 
    return input_str