import logging
import time
import multiprocessing
import sys
import json

logger=logging.getLogger(__name__)

def timeout(func, *args, **kwargs):
    context=kwargs.pop("context", None)
    timeout_seconds=kwargs.pop("timeout_seconds", 60)
    exit_on_timeout=kwargs.pop("exit_on_timeout", False)
    exit_code_on_timeout=kwargs.pop("exit_code_on_timeout", 1)
    format=kwargs.pop("format", 'table')
    if context:
        logger.debug("Start %s with timeout decorator (%s sec)" % (context, timeout))
    p = multiprocessing.Process(target=func, args=args, kwargs=kwargs)
    p.start()
    p.join(timeout_seconds)
    if p.is_alive():
        if context:
            error_msg="Timeout limit reached (%s sec) for %s ..." % (str(timeout_seconds), context)
        else:
            error_msg="Timeout limit reached (%s sec) ..." % str(timeout_seconds)
        if format == "json":
            logger.debug(error_msg)
            logger.info("{\"status\": \"NOK\", \"error\": \"timeout\", \"message\": \"%s\"}" % error_msg)
        else:
            logger.error(error_msg)
        p.terminate()
        p.join()
        if exit_on_timeout:
            sys.exit(exit_code_on_timeout)