import os
import sys
import logging

logger=logging.getLogger(__name__)

def get_binary_version():
    version="0.1.0-SNAPSHOT"
    version_path="files/VERSION" 
    if getattr(sys, 'frozen', False):
        version_path = os.path.join(sys._MEIPASS, "files/VERSION")
    if os.path.isfile(version_path):
        with open(version_path) as f:
            version = f.readline().rstrip()
    return version

def versiontuple(v):
    return tuple(map(int, (v.split("."))))

def check_version(actual_version, version, operator):
    if "SNAPSHOT" in actual_version:
        logger.info("Version is snapshot! It won't compare versions.")
        return
    else:
        actual_version_tup=versiontuple(actual_version)
        version_tup=versiontuple(version)
        result=False
        if operator == "eq":
            result=(actual_version_tup == version_tup)
        elif operator == "ne":
            result=(actual_version_tup != version_tup)
        elif operator == "ge":
            result=(actual_version_tup >= version_tup)
        elif operator == "gt":
            result=(actual_version_tup > version_tup)
        elif operator == "le":
            result=(actual_version_tup <= version_tup)
        elif operator == "lt":
            result=(actual_version_tup < version_tup)
        if result:
            logger.info("Version check successful.")
        else:
            logger.error("Version check failed.")
            sys.exit(1)