# CDP doctor 

CDP doctor is a tool for common checks on FreeIPA, DataLake and DataHub nodes. It is subcommand of `cdp-telemetry` which is available as `cdp-doctor` (it's an alias of `cdp-telemeyry doctor`)

## Developement

You need to start to look `cdp_telemetry/cli.py` and look for the `doctor` command group.
The logic of those doctor commands are located in `cdp_telemetry/doctor` module. For many commands, you can find a few helpful input samples in `examples` folder.

## Avaiable commands

### Network checks

DNS checks, CCM and different endpoints (databus / compute monitoring / cloud provider related) availability (e.g. with proxy) can be done by the following command:

```bash
cdp-doctor network status
```

### Service checks

You can do checks against expected infra services are running on the nodes (service list is different per cluster shapes, but the list of services can be provided by a parameter as well)

```bash
cdp-doctor service status
```

### Metering checks

If metering data is missing for a cluster, worth to do checks against metering related services and configs, that can be done with the following command:

```bash
cdp-doctor metering status
```

### Monitoring checks

Monitoring related checks can be executed by the following command:

```bash
cdp-doctor monitoring status
```

The execution logic relies on `cdp-promethues` and `cdp-request-signer` related configurations.

### Fluentd checks

Fluentd related prcesses and buffers can be observed via this `cdp-doctor` command:

```bash
cdp-doctor fluentd status
```

### System metrics / process checks

There are 3 useful system checks (as subcommands) for `cdp-doctor` inside `system` command group.

First one could be useful to see command system metrics:

```bash
cdp-doctor system metrics
```

Second one could be useful to see how much resources the top processes are consuming:

```bash
cdp-doctor system process
```

Third one could be useful to see more details about network usage (connections/IPs/ports owned by processes):

```bash
cdp-doctor system connections -v
```

### Recipe checks

Recipe results can be checked by the following command (useful on DataLake/DataHub nodes):

```bash
cdp-doctor recipe results
```

You can also fetch more detailed recipe data (by recipe name, e.g.: post-cloudera-manager-start/example-recipe.log)

```bash
cdp-doctor recipe describe --name <recipe name>
```

### Parcel checks

You can check parcel availability with the following command:

```bash
cdp-doctor parcel status -i <imageId>
```

Note that you need internet connection to executing this command, as it tries to download the image catalogs during the check.

### Cloudera agent command checks

To gather command details (like results/names/ids):

```bash
cdp-doctor scm list-commands
```

Describe specific commands (can be filtered on ids/names that can be gathered by list-commands):

```bash
cdp-doctor scm describe-command --id <command id>
```

### Command reports

You can also execute user defined commands and save the outputs to different files. The commands and the outputs can be defined in YAML file.:

```bash
cdp-doctor commands create --config mycommands.yaml
```

```yaml
commands:
    - command: ps aux
      output: ps_aux_command.txt
    - command: nslookup $(hostname -i)
      output: r_lookup_output.txt
    - command: nslookup $(hostname -f)
      output: f_lookup_output.txt
    - command: cdp-doctor metering status
      output: cdp_doctor_metering_output.txt
```

