# Cdp Node Status Monitor

## Description

Node status monitor is simple flask server that is a part of `cdp-telemetry` binary, that can run specific commands in the background. It also serves the collected results that are stored in local static files. At the moment it collect command outputs by salt (with the `cdp-telemetry nodestatus collect` or `cdp-nodestatus collect` (as an alias) commands)

## Configuration

### `server`

Web server related configurations.

#### `server.port`

Port that is used by the web server.

#### `server.folder`

The path where the static files are stored that are served by the web server.

#### `server.auth`

Authentication related configuration for the web server. (authentication details are generated by `Cloudbreak` and stored in `Stack` or `Cluster` entities as `Secret` values)

##### `server.auth.username`

Username that can query the web server.

##### `server.auth.password`

Password for the user.

##### `server.auth.hash`

Hash algorithm that is used for the password

### `commands`

List of commands that are executed by node status monitor. (scheduling is defined by `firstSleep` and `sleepTime` configurations)

### `firstSleep`

It defines how much the application should wait (in seconds) after startup before start executing the commands.

### `sleepTime`

Sleep time (in seconds) between executing the list of commands that are defined in the `commands` config option.

### `saltPing`

It's a specific configuration that will execute ping operation on salt master in order to check the minions.

#### `saltPing.enabled`

Flag that can be turn on/off the salt ping operation.

#### `saltPing.firstSleep`

It defines how much the application should wait (in seconds) after startup before executing the first salt ping.

#### `saltPing.sleepTime`

Sleep time (in seconds) between executing salt ping operations.

### `saltPingCacheEvictSeconds`

Salt ping responses are cached for this amount of time. (therefore if this value is `20`, that means you will got the same result if you hit the salt ping api of node status monitor if the difference between the calls are less than 20 seconds)

### Example

```yaml
server:
  port: 61888
  folder: /var/lib/cdp-nodestatus/report
  auth:
    username: 911fe4df-dab7-43a5-b412-8b00ee823715
    password: c4ae419b6e1fb4839abf35cc20681e6a3847a79a9f1ddda2184785b046b59490
    hash: sha256
commands:
  - "cdp-nodestatus collect --databus-s3-url https://cloudera-dbus-dev.s3.amazonaws.com --databus-url https://dbusapi.sigma-dev.cloudera.com --cloud-provider AWS"
firstSleep: 660
sleepTime: 3600
saltPing:
  enabled: True
  firstSleep: 120
  sleepTime: 1200
saltPingCacheEvictSeconds: 20
```

## CB side configuration

`cdp-nodestatus-monitor` is configured by salt in `nodestatus` state.

## Models

All the models that are used by the application are also defined in proto files, located at `proto/nodestatus.proto`. API json responses should match with these models. On CB side, this proto file is also stored on CB side (that can be used to parse JSON text inputs from the nodestatus monitor API and convert them into Java objects)

## Local developement

```bash
# if you have a virtualenv - see more in the makefile
source env1
python3 cdp_telemetry/cli nodestatus server -c examples/nodestatus/nodestatus-monitor.yaml
```

Note that local development could be limited as you could not have salt locally. (TODO: setup docker environment with salt + nodestatus monitor)

## Debug

![](images/node-status-monitor.gif)

Asciinema cast file: [here](cast/node-status-monitor.cast)

## Release

As node status monitor is a part of `cdp-telemetry`, first you need a release of `cdp-telemetry` itself. Then update the version in `cdp-infra-tools` and release that as well. (from that point, the newest release will be burned in `cloudbreak-images`)

## TODO

This component gathers data through salt and only have server on the master nodes. It would be wise to have agents as well on every node, and gather data through those to keep salt resource load lower. As nodestatus monitor has proto definitions, it should be an easy task to create agents where you can define the expected responses. (although encryption should be solved for the server/agent communication)

