# Filecollector

## Description

Filecollector (`cdp-telemetry` command group) is a component that is able to collect and redact files locally and save the (compressed) output into a specified location. 

Every collection has one or more generated workspace where the data will be collected. Workspace name contains a timestamp and a suffix (index - based on how many workspaces are created for a specific collection). The timestamp (date) value of the name is unique for a collection. 

The application has 2 implementation, first there is an old one written in python (`cdp_telemetry/filecollector` folder) and there is a newer one in written in Go. (it was required to re-implement the application for performance reasons). Currently the application is started by the python application (as that is the basic binary by `cdp-telemetry`), and the inputs are passed to the Go one. The `filecollector` binary itself can be also called directly but note that you need to load `libpcre.so.3.13.3` for the application to run. (Centos 7 had some memory usage related bugs for the built-in pcre library, so a newer pcre lib is bundled with the application)

## Usage

```bash
filecollector --config filecollector.yaml
```

CLI inputs:

- `--config`: main configuration file for the file collection
- `--labels`: Filter input collection by labels (from the file entries that are defined in the configuration file)
- `--start-time`: Collection will include this files, if their timestamp (create / update) is older than this epoch timestamp.
- `--end-time`: Collection will include this files, if their timestamp (create / update) is younger than this epoch timestamp.

### Run / test locally

```bash
cd filecollector
main.go --config ../examples/filecollector/filecollector.yaml
```

## Configuration

### `server`

Filecollector can start a simple http file server. The goal is to serve the files that are collected. By default that feature is not used by CB.

#### `server.port`

Port of the web server.

#### `server.folder`

Folder that is served by the web server.

### `collector`

Collector related configurations that controls how the filecollector application should behave during data collection.

#### `collector.outputLocation`

The output directory where the files will be collected.

#### `collector.files`

List of files that will be processed by the filecollector. Default value: `[]`

##### `collector.files.label`

This value is used as an identifier of the collected file(s) which is used as folders or compressed file name inside the generated bundle. (e.g.: for `mylog`: `<outputLocation>/mylog/whatever.log` or `<outputLocation>/mylog.tar.gz`) 

##### `collector.files.path`

The path (glob) that is the location of the files that will be collected.

##### `collector.files.excludes`

Glob patterns that will be excluded from the file collection.

##### `collector.files.compress`

If this value is set, the collected files for a label will be put into a compressed file with the name of the label and with the extension of the compress format. (a collection with `mylabel` label: `<outputLocation>/mylabel.tar.gz`)

##### `collector.files.compressAllFiles`

If this value is set, every log file itself inside the label folder will be also compressed. That means if the path glob of the log type will match to multiple files, all of the matched files will be compressed.

##### `collector.files.useFullPath`

If this value is set, the path of the collected files will be fully included. (e.g.: `/var/log/cdp-logging-agent/cdp-logging-agent.log` -> `<outputLocation>/<log type>/var/log/cdp-logging-agent/cdp-logging-agent.log`) This setting overrides `collector.useFullPath`. (so this setting can also disable that configuration for a specific label)

##### `collector.files.folderPrefix`

Add an extra directory prefix for the label. So instead of put the collected files for a label into the bundle like this: `<outputLocation>/<label>`, the collected files will be inside `<outputLocation>/<folderPrefix>/<label>`

##### `collector.files.skipLabelFromPath`

If this value is set, the label will be skipped from the output for the file collection. So the generated output will look like `<outputLocation>/...` instead of `<outputLocation>/<label>`. Default value: `false`

##### `collector.files.skipAnonymization`

If this value is set, none of the `collector.rules` will be used for redaction of the logs of this file entry. Default value: `false`

##### `collector.files.mandatory`

If the file entry is flagged with this value, then the collection cannot be skipped by filtering labals. Default value: `false` 

#### `collector.rules`

List of anonymization rules, that will be applied on every processed file (unless `skipAnonymization` is set for the log type). Default value: `[]`

##### `collector.rules.pattern`

A `PCRE` regex pattern that is matched against every line of the processed logs. (Cannot be empty)

##### `collector.rules.replacement`

In case of a match (based on `collector.rules.pattern`), the matched value will be replaced with this one. Default value: `[REDACTED]`

##### `collector.rules.labels`

The list of labels (log type filtring) where the pattern matching will be used. Default value: `[]` 

#### `collector.fileRelativePrefix`

Can be used to add an extra prefix for the collected files. (a collected file output: `<outputLocation>/<fileRelativePrefix>/<collected file path>`)

#### `collector.requiredDiskSpaceRatio`

Required disk space ratio for the data collection. Ratio is against the size of the data that will be collected. So if the data size of the collected files are `100MB` and the required disk space ratio is `1.2`, that means the application will require at least `120MB` free disk space, otherwise, the collection will fail. Default value: `1.0`

#### `collector.requiredFreeMemoryRatio`

Defines ratio regarding free memory - that is needed for the application to run the collection. The expected / used memory can be calculated based on the `bytesBufferSizeForAnonymiziation`. (per worker) The value is used the same way as `requiredDiskSpaceRatio` above. Default value: `1.3`

#### `collector.bytesBufferSizeForAnonymiziation`

The size of the buffer that is used by the application during anonymization.

#### `collector.fileLimitBytes`

File size limit in bytes. If a file size reaches this value, the data file will be "tailed". (e.g.: old `salt-master` logs were not rotated, in the file is too large only the last n line will be processed) Default value: `104857600`

#### `collector.numberOfLinesToKeepOnLimit`

The number of lines to keep for a file in case of hitting the limit defined in `fileLimitBytes` configuration. Default value: `1000000`

#### `collector.workspaceWorkers`

Number of workspace workers. It is used for parallelization for data processing. If more than 1 worker is used, it will put different log types into different workspaces. Default value: `1`

#### `collector.workspaceLimitBytes`

Limit of the workspace size in bytes. If the value is reached a new workspace will be used with a +1 index in the suffix of the workspace name (if it is not 0). Default value: `0`

#### `collector.workspaceMetadataFileName`

Name of the workspace metadata. The file contains useful metadata information about the processed / collected files. Default value: `filecollector-metadata.json`

#### `includeWorkspaceMetadata`

If it is set, include workspace metadata file in the bundle. Default value: `true`

#### `collector.additionalWorkspaceMetadata`

YAML object type that will be included in the bundle (with file name that is defined in `additionalWorkspaceMetadataFileName`) in a JSON format. 

#### `collector.additionalWorkspaceMetadataFileName`

Name of the additional metadata file. Default value: `bundle-info.json`

#### `collector.useFullPath`

If this value is set, the path of the collected files will be fully included. (e.g.: `/var/log/cdp-logging-agent/cdp-logging-agent.log` -> `<outputLocation>/<log type>/var/log/cdp-logging-agent/cdp-logging-agent.log`) Default value: `true`

#### `collector.compress`

If it is set, the workspace will be compressed. Default value: `true`

#### `collector.compressFormat`

Compression format. Default value: `zip` (Available values: `tar`, `gztar`, `bztar`, `zip`)

#### `collector.deleteProcessedTempFiles`

Deletes collected workspace files and folders (these files will be compressed if `collector.compress` is set). Useful for turn this off for debugging. Default value: `true`

#### `collector.logProcessorWorkers`

Number of log processors. For every workspace there are `n` number of logs that needs to be processed / redacted. By increasing the number of log processors you can increase the speed of data processing (but using more memory) as if the value is larger than 1, the application can process multiple log files together. Default value: `1`

#### `collector.freePageCacheOnStart`

If this is set free page cache (memory) before the collection starts. It will run the follwoing command: `echo 1 > /proc/sys/vm/drop_caches` (unsafe). Default value: `false`

#### `collector.freePageCacheOnFinish`

If this is set free page cache (memory) after the collection finished. It will run the follwoing command: `echo 1 > /proc/sys/vm/drop_caches` (unsafe). Default value: `false`

## Debug

On VMs, the best place to look for detils about the collection or troubleshooting is the `/var/log/salt/master` and `/var/log/salt/minion`, or check the application log itself in `/var/log/cdp-telemetry/filecollector.log`. Collected files can be found in `/var/lib/filecollector` and the configurations are located in `/opt/cdp-telemetry/conf` directory.

![](images/filecollector.gif)

Asciinema cast file: [here](cast/filecollector.cast)

## Relationship with diagnostics

In Cloudbreak, `filecollector` is used by the VM diagnostics flow at the collection phase. That is the only phase where filecollector is used. (it is not used in the CM based bundle collection)