package filecollector

import (
	"errors"
	models "filecollector/models"
	processor "filecollector/processor"
	"fmt"
	"os"
	"path/filepath"
	"strings"
	"time"

	"github.com/urfave/cli"
)

func CollectCommand() cli.Command {
	return cli.Command{
		Name:  "collect",
		Usage: "Collect, archive and redact files based on configuration",
		Action: func(c *cli.Context) error {
			config := c.String("config")
			if len(config) == 0 {
				return errors.New("Configuration file argument is required!")
			} else {
				if _, err := os.Stat(config); os.IsNotExist(err) {
					return errors.New(fmt.Sprintf("Configuration error: %s", err))
				}
			}
			startTime := c.Int64("start-time")
			endTime := c.Int64("start-time")
			labelsInput := c.String("labels")
			labels := make([]string, 0)
			if len(labelsInput) > 0 {
				labels = strings.Split(labelsInput, ",")
			}
			userInput := models.UserInput{Labels: labels, FromDate: startTime, ToDate: endTime}
			executableDir, filePathErr := filepath.Abs(filepath.Dir(os.Args[0]))
			if filePathErr != nil {
				return filePathErr
			}
			collectorConfig, parserErr := models.ParseCollectorConfig(config, executableDir)
			if parserErr != nil {
				return parserErr
			}
			start := time.Now()
			fmt.Println("Starting file collection.")
			collectorErr := processor.CollectFiles(collectorConfig, userInput)
			duration := time.Since(start)
			if collectorErr != nil {
				fmt.Println(fmt.Sprintf("FAILED - Execution time: %s", duration))
				return collectorErr
			}
			fmt.Println(fmt.Sprintf("SUCCESS - Execution time: %s", duration))
			return nil
		},
		Flags: []cli.Flag{
			cli.StringFlag{Name: "config, c", Usage: "YAML configuration file for file collection"},
			cli.StringFlag{Name: "labels, l", Usage: "Comma separated labels for filtering"},
			cli.IntFlag{Name: "start-time, s", Usage: "Unix epoh start timestamp for filtering files"},
			cli.IntFlag{Name: "end-time, e", Usage: "Unix epoh end timestamp for filtering files}"},
		},
	}
}
