package filecollector

import (
	"io/ioutil"
	"os"
	"path/filepath"
	"time"

	"github.com/sirupsen/logrus"
	"github.com/snowzach/rotatefilehook"
)

func NewLogger(FileName string) *logrus.Logger {
	logLevel := logrus.DebugLevel
	log := logrus.New()
	log.SetLevel(logLevel)
	logDir := "/var/log/cdp-telemetry"
	logFileName := filepath.Join(logDir, FileName)
	if _, err := os.Stat(logDir); os.IsNotExist(err) {
		logFileName = FileName
	}
	rotateFileHook, err := rotatefilehook.NewRotateFileHook(rotatefilehook.RotateFileConfig{
		Filename:   logFileName,
		MaxSize:    20,
		MaxBackups: 5,
		MaxAge:     28,
		Level:      logLevel,
		Formatter: &logrus.JSONFormatter{
			TimestampFormat: time.RFC822,
		},
	})

	if err != nil {
		logrus.Fatalf("Failed to initialize file rotate hook: %v", err)
	}
	log.SetFormatter(&logrus.TextFormatter{
		PadLevelText:    true,
		ForceColors:     true,
		FullTimestamp:   true,
		TimestampFormat: "2006-01-02 15:04:05",
	})
	log.SetOutput(ioutil.Discard)
	log.AddHook(rotateFileHook)
	return log
}
