package filecollector

import (
	"bufio"
	models "filecollector/models"
	"fmt"
	"io"
	"os"
)

func RedactFile(fileObj models.FileObject, bytesBufferSizeForAnonymiziation int64) error {
	rules := fileObj.Rules
	log.Debug(fmt.Sprintf("Start anonymization for %s", fileObj.DestFileName))
	tempFileName := fileObj.TempFileName
	f, err := os.Open(tempFileName)
	if err != nil {
		return err
	}
	defer f.Close()
	df, destErr := os.Create(fileObj.DestFileName)
	if destErr != nil {
		return destErr
	}
	defer df.Close()

	reader := bufio.NewReader(f)
	buf := make([]byte, bytesBufferSizeForAnonymiziation)
	for {
		n, err := reader.Read(buf)
		if err != nil {
			if err != io.EOF {
				return err
			}
			break
		}
		textInBytes := buf[0:n]
		for _, rule := range rules {
			textInBytes = rule.Regex.ReplaceAll(textInBytes, rule.Replacement, 0)
		}
		df.Write(textInBytes)
	}
	return nil
}
