package proxy

import (
	"errors"
	"fmt"
	"strconv"
	"strings"

	bigcache "github.com/allegro/bigcache/v3"
	log "github.com/sirupsen/logrus"
)

func (client *UMSClient) AddToAuthNCache(token string, actorCrn string) {
	client.authNtokenCache.AddToCache(token, actorCrn)
}

func (client *UMSClient) AddToAuthNFailureCache(token string, errorMsg string, statusCode int) {
	AddToFailureCache(client.authNFailureCache, token, errorMsg, statusCode)
}

func (client *UMSClient) AddToAuthRCache(token string, actorCrn string) {
	client.authRtokenCache.AddToCache(token, actorCrn)
}

func (client *UMSClient) AddToAuthRFailureCache(token string, errorMsg string, statusCode int) {
	AddToFailureCache(client.authRFailureCache, token, errorMsg, statusCode)
}

func AddToFailureCache(tokenCache *CdpTokenCache, token string, errorMsg string, statusCode int) {
	tokenCache.AddToCache(token, fmt.Sprintf("%s ### %d", errorMsg, statusCode))
}

func (client *UMSClient) GetFromAuthNCache(token string) (string, bool) {
	return client.authNtokenCache.GetFromCache(token)
}

func (client *UMSClient) GetFromAuthNFailureCache(token string) (string, int, bool) {
	return ReadFromFailureCache(client.authNFailureCache, token)
}

func (client *UMSClient) GetFromAuthRCache(token string) (string, bool) {
	return client.authRtokenCache.GetFromCache(token)
}

func (client *UMSClient) GetFromAuthRFailureCache(token string) (string, int, bool) {
	return ReadFromFailureCache(client.authRFailureCache, token)
}

func (tokenCache *CdpTokenCache) AddToCache(token string, value string) {
	cache := tokenCache.cache
	log.Debugf("Add token to %v for value: %v", tokenCache.identifier, value)
	cache.Set(token, []byte(value))
	log.Debugf("Cache '%v' size (ADD): %v", tokenCache.identifier, cache.Len())
}

func (tokenCache *CdpTokenCache) GetFromCache(token string) (string, bool) {
	cache := tokenCache.cache
	resp, err := cache.Get(token)
	log.Debugf("Cache '%v' size (GET): %v", tokenCache.identifier, cache.Len())
	if err != nil {
		if errors.Is(err, bigcache.ErrEntryNotFound) {
			log.Debugf("No entry found for cache request (data could be evicted).")
		} else {
			log.Debugf("Error during gathering data from cache (%s), error: %v", tokenCache.identifier, err)
		}
		return "", false
	}
	return string(resp), true
}

func ReadFromFailureCache(cdpTokenCache *CdpTokenCache, token string) (string, int, bool) {
	fullMsg, found := cdpTokenCache.GetFromCache(token)
	num := 500
	if found {
		splitted := strings.Split(fullMsg, " ### ")
		if len(splitted) > 1 {
			parsedNum, err := strconv.Atoi(splitted[1])
			if err != nil {
				log.Errorf("Cannot parse string as int: %s, error: %v", splitted[1], err)
			}
			num = parsedNum
		}
		return splitted[0], num, true
	}
	return "", num, found
}
