package proxy

import (
	"fmt"
	"net/http"
)

func GetAuthString(r *http.Request) string {
	return getOr(r, "X-Altus-Auth", "X-Cdp-Auth")
}

func GetAuthToken(r *http.Request) string {
	return getOr(r, "X-Altus-Token", "X-Cdp-Token")
}

func GetAuthDate(r *http.Request) string {
	return getOr(r, "X-Altus-Date", "X-Cdp-Date")
}

func GetAuthMsg(r *http.Request) string {
	return getOr(r, "X-Altus-Msg", "X-Cdp-Msg")
}

func GetRequestIdFromHeader(r *http.Request) string {
	return r.Header.Get("X-Cdp-Request-Id")
}

func GetResourceCrnFromHeader(r *http.Request) string {
	return r.Header.Get("X-Cdp-Resource-Crn")
}

func GetInstanceFromHeader(r *http.Request) string {
	return r.Header.Get("X-Cdp-Instance")
}

func GetRequestHeaderInfo(r *http.Request) string {
	result := ""
	requestId := GetRequestIdFromHeader(r)
	if len(requestId) > 0 {
		result += fmt.Sprintf(" - requestId: %v", requestId)
	}
	resourceCrn := GetResourceCrnFromHeader(r)
	if len(resourceCrn) > 0 {
		result += fmt.Sprintf(" - resourceCrn: %v", resourceCrn)
	}
	instanceName := GetInstanceFromHeader(r)
	if len(instanceName) > 0 {
		result += fmt.Sprintf(" - instance: %v", instanceName)
	}
	if len(result) > 0 {
		return fmt.Sprintf(" Header info:%v", result)
	}
	return result
}

func getOr(r *http.Request, header1 string, header2 string) string {
	header1Val := r.Header.Get(header1)
	if len(header1Val) == 0 {
		return r.Header.Get(header2)
	}
	return header1Val
}
