# CDP Logging Agent

## Description

`cdp-logging-agent` is a re-packaged / rebuilt fluentd with pre-installed custom plugins (including ruby that has been built from source)

## Build

```bash
mvn -pl cdp-logging-agent clean package 
```

The build should produce a signed RPM from fluentd with custom plugins. During the build, multiple containers are produced. (all the references below are located in files inside `cdp-logging-agent` directory)

First, from `Docker.base` a specific fluentd is built from a specific version of ruby. (see the following docker args: `FLUENTD_REVISION`, `BUNDLED_RUBY_FEATURE_VERSION` and `BUNDLED_RUBY_VERSION` in the dockerfile). If a new `fluentd` version is needed, update the revision and the base image version. (these are located in the `Makefile` of the cdp-logging-agent project, but the defaults can be updated in the `Dockerfile` as well). This step can be triggered manually by `make build-base-image` command.

Secondly, `Docker.pkg` is built in order to install all the pre-defined gems / plugins. The gem versions are defined in `src/main/resources/gems.json` (in case of fixing CVE issues, gems can be updated here). This task can be triggered by `make build-for-vm`

The above 2 docker containers are pushed (newly) to the docker registry only if a new version is set. (both in parent `pom.xml` and `re_configs/globals.ini`). If the version is not moved, it will only pull the actual version. The reason for this is that the build would take 1h+ because of the ruby and fluentd build. (so it can be changed to always build the fluentd+ruby with the custom gems, but it has a cost, ideally these should rearly change)

In the end, rpm packages will be generated by `fpm` (see: https://github.com/jordansissel/fpm) image. Above builder images will generate a tar.gz file with the content that will be packaged to an RPM.

Note: The cp and vm prefixes in the `Makefile` refering to control plane and virtual machines. Generated docker image can be re-used on the Control Plane, RPM one can be used for VMs. (they are installing different gems based on the profiles in the `Makefile`)

## Update plugins

See more about updating / releasing plugins here: [wiki](https://cloudera.atlassian.net/wiki/spaces/ENG/pages/771982673/Release+process+of+FluentD+plugins+CP+FluentD+container)
Note: for testing, the databus plugin is the most developer friendly.

## QA environment

For QA environment, in order to make the databus plugin work, there is a machine user in mow-dev UMS called `cb-e2e-fluent-databus-user` and its secrets are used by the QA Cloudbreak services.

## Increase versions

Increase version in both parent `pom.xml` and `re_config/globals.ini` before releasing. (if using new version of cdp-logging-agent is needed)

## Release

See cdp-infra-tools release process in `README.md`.

## CB side definitions

CB side salt states for the logging-agent is located at `salt-common/fluent` folder.

## Burn cdp-logging-agent on VM images

Once cdp-logging-agent is released (as the new rpm will be avialable at: [https://archive.cloudera.com/cdp-infra-tools/latest/redhat7/](https://archive.cloudera.com/cdp-infra-tools/latest/redhat7/)). In cloudbreak-images project, always the latest released version is installed by this script: [https://github.com/hortonworks/cloudbreak-images/blob/master/saltstack/base/salt/telemetry/centos/scripts/install-package.sh](https://github.com/hortonworks/cloudbreak-images/blob/master/saltstack/base/salt/telemetry/centos/scripts/install-package.sh)

## Upgrade on VMs

The package for VMs can be installed from `cdp-infra-tools` repository. It always (!) points to the latest.

```bash
yum update -y --disablerepo=* --enablerepo=cdp-infra-tools cdp-logging-agent
```

or you can install snapshots from releng builds like this:

```bash
yum remove -y cdp-logging-agent
rpm -i https://cloudera-build-us-west-1.vpc.cloudera.com/s3/build/32372097/cdp-infra-tools/0.x/redhat7/yum/cdp_logging_agent-0.3.6.x86_64.rpm # or scp it to a VM if not available from the node
/opt/salt_*/bin/salt-call state.apply fluent.init
```
 
## Debug

### Logging agent process issue

In case of any startup issue, check `journalctl -u cdp-logging-agent` first, then for runtime issues, look for `/var/log/cdp-logging-agent/cdp-logging-agent.log`

![](images/startup-issue.gif)

Asciinema cast file: [here](cast/startup-issue.cast)

As you can see in the above example, you can edit ruby files and restart cdp-logging-agent with the change (for debugging).

### Debug with metering databus processing

As cdp-logging-agent is responsible to process metering heartbeats by databus plugin, this could be the most critical part of the application that should work properly.

Here is a simple example about a typical debug on a VM node:

![](images/metering-databus.gif)

Asciinema cast file: [here](cast/metering-databus.cast)

Overall, with any logging agent related issues, the easiest way to debug is to stop the systemd service and start logging agent in foregroud by `cdp-logging-agent` command. (same as in the above example)

more metering related troubleshooting: [wiki](https://cloudera.atlassian.net/wiki/spaces/ENG/pages/1057062919/DataHub+Metering+-+troubleshooting+on+VM+nodes)

### Position and buffer file locations

Both the position files and the buffer files are located at `/var/log/cdp-logging-agent`

![](images/pos-and-buffers.gif)

Asciinema cast file: [here](cast/pos-and-buffers.cast)

### Related `cdp-doctor` checks

The following commands could be useful with `cdp-doctor` for troubleshooting:

- `cdp-doctor fluentd status`
- `cdp-doctor metering status`
- `cdp-doctor network status`

### Diagnostics

In case of `cdp-telemetry` is available on the nodes and `salt` components are stable, you use the collect diagnostics by `cdpcli` (or `UDX` UI), in the bundle the content of the `report` folder or `logs/fluentd.tar.gz` file could be useful.

### Configuration

Read more about fluentd configuration at [https://docs.fluentd.org/configuration](https://docs.fluentd.org/configuration)

