# CDP UMS Auth Proxy

## Description

CDP UMS Auth Proxy is a simple (reverse) proxy that can be used authenticate incoming requests against UMS by provided custom headers (against authentication strings or signed cacheable tokens).

## Usage

Pull the docker container and run it with a provided configuration.

```bash
docker pull docker-private.infra.cloudera.com/cloudera/cdp-ums-auth-proxy:1.2.3-b1
docker start -v /my/path/cdp-ums-auth-proxy.yaml:/etc/cdp-ums-auth-proxy/cdp-ums-auth-proxy.yaml docker-private.infra.cloudera.com/cloudera/cdp-ums-auth-proxy:1.2.3-b1
```

It is important for the application to resolve UMS host from the container, otherwise incoming requests will fail.

## Configuration sections

Configuration file (YAML) can be set by `--config` argument for the application. The containerized format of the server uses `/etc/cdp-ums-auth-proxy/cdp-ums-auth-proxy.yaml` by default. 

Example: 

```yaml
port: 61000
ums:
  host: ums-mock
  port: 61982
cache:
  ttlMinutes: 5
  ttlFailureSeconds: 20
  maxSizeMB: 8912
  shards: 256
  cleanTimeMinutes: 1
logger:
  level: debug
authorization: 
  enabled: true
  rights:
    - "monitoring/pushComputeMetric"
  verifyAccountId: true
  originalUriHeaderName: X-Original-URI
```

#### `port`

Port that is allocated for the application.

#### `ums`

UMS related configurations.

- `host`: Host of the UMS server (grpc endpoint)
- `port`: Port of the UMS server
- `connectionTimeoutSeconds`: Grpc client connection timeout in seconds. (default: `5`)
- `requestTimeoutThresholdSeconds`: Request time threshold in seconds. Log warnings if this threashold is reached during processing a request. (default: `5`)

#### `cache`

Cache (AuthN/R) related configurations. The cache is storing key-value pairs. (key: a request token, value: a machine user name). It is only used if the incoming requests contain `x-altus-token` or `x-cdp-token` header (otherwise signed requests will be always validated against UMS).

- `ttlMinutes`: Life window in minutes of the cached (successful AuthN/R) elements. (default: `90`)
- `ttlFailureSeconds`: Life window in seconds of the cached (failed AuthN/R) elements. (default: `20`)
- `maxSizeMB`: Maximum size of cache. Oldest elements will be dropped after the size is reached. (default: `8912`)
- `shards`: Number of shards in the cache(s). Must be a power of 2. (defalt: `256`)
- `cleanTimeMinutes`: Cache cleanup period in minutes. Remove elements that are reached its TTL window limit. (default: `5`)

#### `logger`

Logger related configurations.

- `level`: Level of the logging (default: `debug`)
- `file`: If it is set, logs will directed into `/var/log/cdp-ums-auth-proxy/cdp-ums-auth-proxy.log` (default: `false`)

#### `authorization`

Authorization (UMS) related configurations.

- `enabled`: Set to `true` in order to enable authorization (default: `false`)
- `rights`: List of rights that the machine user needs to be able to process the remote writes. Recommended value: `monitoring/pushComputeMetric`
- `verifyAccountId`: Verify account ID against the authenticated machine user. If this is enabled, a machine user will be only authorized, if the original uri contains the account ID of the user. (default: `true`)
- `originalUriHeaderName`: Original URI header that can be passed by proxies, and that header will be used during account ID verification. (default: `X-Original-URI`)

## Environment variables

All the configurations can be set by environment variables as well. (but with uppercase format with `_` separator instead of `.`, e.g.: `ums.host` -> `UMS_HOST`). For container based deployements, use environment variables in order to override only the non-default parameters.

```yaml
environment:
  - UMS_HOST=umshost
  - UMS_PORT=1234
```

Note: string lists as environment variables - like `AUTHORIZATIOn_RIGHTS` - can be passed as space (!) separated variables.

## Debug on monitoring Kubernetes environment

![](images/ums-auth-proxy.gif)

Asciinema cast file: [here](cast/ums-auth-proxy.cast)

## Integration testing

In `integration-test` module, you can run tests by using:

```bash
make test
```

Or if you do not want to validate the results of the tests, but you would like to setup the docker-based environment, and observing the container manually you can also just do this:

```bash
make run
```

### Available releases

[RelEng portal link](https://release.infra.cloudera.com/stackspage?stack=CDP-INFRA-TOOLS&filter=SHIPPED)
