# Usage Processor - DEPRECATED

The building of `cdp-telemetry-usage-processor` image (usage-processor) was moved to the [thirdparty_images](https://github.infra.cloudera.com/docker/thirdparty_images/tree/master/cdp-telemetry-usage-processor) repository.
This descripton below is also outdated.

# DEPRECATED

## Description

Usage processor is component that is reponsible to serve events based on incoming HTTP requests (body) and delegate/transform data direclty for cloudwatch. The processor itself is a `fluent-bit` that is built by `cdp-logging-agent-bit` project.

The current `usage-processor` was built in [cdp-telemetry-cli](https://github.infra.cloudera.com/thunderhead/cdp-telemetry-cli) as an infra container, but use `cdp-logging-agent-bit` in the future. (as on release, that one is published to production ECR)

## Build

The build should only produce a docker container.

```bash
mvn -pl cdp-logging-agent-bit clean package
```

Image is created from `aws-observability/aws-for-fluent-bit:2.26.0`. (It contains custom aws fluent-bit plugins)

## Control Plane

Usage processor helm chart definition is located at `dps-k8s/cloudbreak/helm-charts/cloudbreak-umbrella/charts/usage-processor`. On `dps` cluster (QA environments) it uses an aws access/secret keypair from the `mowdev` user. (therefore it uses cloudwatch log group on `mow-dev`, on mow environments, the AWS service access is role based).

## Configuration

### Control Plane configuration

As the `usage-processor` is a `fluent-bit`, you can see mroe about its configuration in [https://docs.fluentbit.io/manual/administration/configuring-fluent-bit/classic-mode/configuration-file](https://docs.fluentbit.io/manual/administration/configuring-fluent-bit/classic-mode/configuration-file)

Current configuration contains an `http` input on port `61890` in order to service requests (coming from any CB service), a filter that modifies the records to include `@env` and `@cluster` fields in the `cloudwatch` outputs. (`cloudwatch_logs` in the output configuration is a plugin that is provided by AWS, not by fluent-bit itself, as a default plugin, one significant different is you can provide dynamic values for output configurations - e.g. like a calculated field in a filter can be used as an output parameter as: `log_group_name: $(my_custom_field)`)

```text
    [SERVICE]
        daemon                     off
        log_level                  debug
        flush                      1
        storage.path               /var/log/flb-storage
        storage.sync               normal
        storage.checksum           off
        storage.backlog.mem_limit  20M
    [INPUT]
        name                http
        host                0.0.0.0
        port                61890
        buffer_max_size     200M
        buffer_chunk_size   4M
        storage.type        filesystem
    [FILTER]
        Name record_modifier
        Match *
        Record @env ${ENVIRONMENT}
        Record @cluster ${CLUSTER}
        Record cloudwatch_group /${ENVIRONMENT}/${CLUSTER}
        Record cloudwatch_stream ${STREAM_PREFIX}_${NODE_NAME}
    [OUTPUT]
        Name cloudwatch_logs
        Match *
        log_stream_name ${STREAM_PREFIX}_${NODE_NAME}
        log_group_name /${ENVIRONMENT}/${CLUSTER}
        region ${AWS_REGION}
        log_format json/emf
```

### Cloudbreak side configuration

Cloudbreak related configuration can be found in `common-config.yaml` under `telemetry.usage.http` config section.

### Relationship with message broker

As you can see on CB side, in `common-config.yaml` the is also a `messagebroker` config section under `telemetry.usage`. With message broker, you can also send usage data to EDH (but only if you set its processor header to `EDH` from `NO_EDH`, see more: [here](https://cloudera.atlassian.net/wiki/spaces/ENG/pages/1207140597/EDH+adding+tracking+for+a+feature#Sending-events-to-dev-EDH-from-local-CB)). The `messagebroker` service itself is a `databus` service, so it has a kinesis consumer / producer. From `cloudbreak` service usages can be sent to databus GRPC endpoint. (can be followed by `SigmaDatabusClient` logs). Although usages are sent to both message broker and usage processor, the messge broker one is only informative for the message broker service, and data won't end in EDH. (because of that `NO_EDH` header) The main reason for that is to avoid dependency on `docker` logs for the usage processing. (message broker writes incoming databus record to logs, and that is the main source of the usage, therefore if log parsing is broken, usages won't end up in EDH) 

## Debug

The usage-component is relatively stable, mostly AWS cloudwatch access misconfiguration that can cause problems. (missing access/secret key on dps or IAM role / service account problems on mow environments)

![](images/usage-processor.gif)

Asciinema cast file: [here](cast/usage-processor.cast)

## TODO

Install aws cli inside the container or use an awscli container for readiness/liveness check. (currently used container has that, but probably the one which is produced by cdp-infra-tools do not have - commented out in the Dockerfile)
