package mock

import (
	"context"
	"fmt"
	"log"

	"github.infra.cloudera.com/ums-mock/generated/usermanagement"
	"google.golang.org/grpc/metadata"
)

type MockUMSServer struct {
	usermanagement.UserManagementServer
	usermanagement.AuthorizationServer
}

func (m *MockUMSServer) Authenticate(ctx context.Context, req *usermanagement.AuthenticateRequest) (*usermanagement.AuthenticateResponse, error) {
	md, ok := metadata.FromIncomingContext(ctx)
	if ok {
		requestId := md.Get("requestID")
		actorCrn := md.Get("actorCrn")
		log.Println(fmt.Sprintf("[Authenticate] Incoming requestId: %v", requestId))
		log.Println(fmt.Sprintf("[Authenticate] Incoming actorCrn: %v", actorCrn))
	}
	response := usermanagement.AuthenticateResponse{ActorCrn: "crn:altus:iam:us-west-1:altus:user:__internal__actor__"}
	log.Println(fmt.Sprintf("Response for authenticate operation: %v", response))
	return &response, nil
}

func (m *MockUMSServer) GetVersion(ctx context.Context, req *usermanagement.VersionRequest) (*usermanagement.VersionResponse, error) {
	log.Println(fmt.Sprintf("Calling getVersion operation: %v", req))
	return nil, nil
}

func (m *MockUMSServer) HasRights(ctx context.Context, req *usermanagement.HasRightsRequest) (*usermanagement.HasRightsResponse, error) {
	md, ok := metadata.FromIncomingContext(ctx)
	if ok {
		requestId := md.Get("requestID")
		actorCrn := md.Get("actorCrn")
		log.Println(fmt.Sprintf("[HasRights] Incoming requestId: %v", requestId))
		log.Println(fmt.Sprintf("[HasRights] Incoming actorCrn: %v", actorCrn))
	}
	numOfChecks := len(req.Check)
	results := make([]bool, 0)
	for i := 0; i < numOfChecks; i++ {
		results = append(results, true)
	}
	response := usermanagement.HasRightsResponse{Result: results}
	log.Println(fmt.Sprintf("Response for hasRights operation: %v", response))
	return &response, nil
}
