#!/bin/sh

main() {
    local file_location=${1:?"usage: <file_location>"}
    grep -qxF "_source_filedigest_algorithm 8" ${HOME}/.rpmmacros || echo "_source_filedigest_algorithm 8" >> ${HOME}/.rpmmacros
    grep -qxF "_binary_filedigest_algorithm 8" ${HOME}/.rpmmacros || echo "_binary_filedigest_algorithm 8" >> ${HOME}/.rpmmacros
    echo "SHA256 before RPM sign:"
    sha256sum $file_location
    echo "RPM version:"
    rpm --version
    echo "RPM macros:"
    cat ${HOME}/.rpmmacros
    local gpg_pass_for_sign=$(sed -n 's:.*<gpg\.passphrase>\(.*\)</gpg\.passphrase>.*:\1:p' ${HOME}/.m2/settings.xml | tr -d \\n)

    cat - > "/tmp/sign_rpm_non_interactive.sh" <<EOF
#!/usr/bin/expect -f
set timeout 600
spawn rpm --addsign ${file_location}
expect "Enter pass phrase:"
send "${gpg_pass_for_sign}\r"
expect ".*"
set outcome $expect_out(buffer)
puts $outcome ""
expect eof

EOF
    chmod +x /tmp/sign_rpm_non_interactive.sh
    echo "RPM sign output: "
    /tmp/sign_rpm_non_interactive.sh
    echo "SHA256 after RPM sign:"
    sha256sum $file_location
    echo "Check RPM sign:"
    rpm -q --qf '%{SIGPGP:pgpsig} %{SIGGPG:pgpsig}\n' -p $file_location
    echo "RPM details:"
    rpm -qpi $file_location
}

if [ "$(uname -s)" = 'Linux' ]; then
  main ${1+"$@"}
fi